/* Copyright (c) 2001-2005 Todd C. Stellanova, rawthought
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The  above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE. 
 * 
 *
 * Created on November 6, 2002, 11:26 AM
 */

package org.kablog.kgui;

import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;

import org.kablog.xmlrpc.KXmlRpcFault;


/**
 *
 * @author  Todd C. Stellanova
 * @version
 */
public class KNetworkProgress extends Form implements Runnable, CommandListener {
    
    protected static final long PROGRESS_DELAY_TIME= 1000;
    
    Gauge progressGauge;
    Thread myThread;
    boolean bCancelled = false;
    KNetworkClient target;
    Command cmd_Cancel;
    Displayable parent;
    Display display;
    StringItem msgDisplay;
    boolean debugOn = false;
    
    public KNetworkProgress(String title, String msg, KNetworkClient aTarget, Displayable owner, Display curDisplay) {
        super(title);
        msgDisplay = new StringItem(msg,null);
        append(msgDisplay);
        progressGauge = new Gauge("Progress:",false,0,100);
        append(progressGauge);
        target = aTarget;
        cmd_Cancel = new Command("Cancel",Command.CANCEL,1);
        addCommand(cmd_Cancel);
        parent = owner;
        display = curDisplay;
        setCommandListener(this);
    }//KNetworkProgress
    
    public void start() {
        this.bCancelled = false;
        display.setCurrent(this);
        myThread = new Thread(target);
        myThread.start();

        display.callSerially(this); //start checking progress periodically
        
    }//start
    
    public void cancel() {
        this.bCancelled = true;
    }//cancel
    
    /**
     * @return True if we should continue looping, false otherwise.
     */
    protected boolean checkProgress() {
        boolean continueChecking = false;
 
        int errCode = target.getFailureCode();
        
        if (errCode != 0) {
            Object asyncResult = target.getAsyncResult();
            if (asyncResult instanceof KXmlRpcFault) {
                String faultString = ((KXmlRpcFault)asyncResult).getMessage();
                msgDisplay.setText(faultString);
            }
            else
                msgDisplay.setText("Failed with err code: " +errCode);
        }
        else {
            int progress = target.getProgress();
            progressGauge.setValue(progress);
            if (progress != 100)
                continueChecking = true;
        }
 
        return continueChecking;
    }//checkProgress
    
    /**
     * We should get this when the user clicks "Cancel"
     */
    public void commandAction(Command c, Displayable d) {
        if (c == cmd_Cancel) {
            cancel();
        }
        else {
           // if (debugOn)
           //     System.out.println("KNetworkProgress unhandled cmd: " + c);
        }
    }
    
    public void run() {
        boolean keepChecking = checkProgress();
        if (keepChecking) {
            try { Thread.sleep(PROGRESS_DELAY_TIME);}
            catch (InterruptedException intEx) {}
            display.callSerially(this);
        }
        else {
            display.setCurrent(parent);//return control to parent
        }
    }
    
//commandAction
    
//cancel
}//class KNetworkProgress
