/* Copyright (c) 2001-2005 Todd C. Stellanova, rawthought
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The  above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE. 
 * 
 *
 * Created on November 6, 2002, 11:48 AM
 */

package org.kablog.kgui;

/**
 * Interface for exporting the progress of a given item
 * @author  Todd C. Stellanova
 */
public interface KProgressStatus {
    
    public static final int ERR_NO_NETWORK = 1015;
    public static final int ERR_BAD_USER   = 1025;
    public static final int ERR_IO         = 1035;
    public static final int ERR_USER_HAS_NO_BLOGS = 1045;

    /**
     * @return The progress of this item, from 0 to 100 
     */
    public int getProgress();
    
    /**
     * Tells the progress generator to reset its progess counter to zero, and clear errs.
     */
    public void reset();
    
    /**
     * @ return Zero if no failure, or the nonzero error code;
     */
    public int getFailureCode();
    
    
    /**
     * @return True if operation can be cancelled, false if not
     */
    public boolean isCancellable();
    
    /**
     * Attempt to cancel the operation in progress. This may not work 
     * if isCancellable returns false.
     */
    public void cancel();
    
} //interface KProgressStatus
