/* Copyright (c) 2001-2005 Todd C. Stellanova, rawthought
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The  above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE. 
 * 
 * 
 */

package org.kablog.kgui;

import java.io.*;
import java.util.*;
import javax.microedition.io.*;
import javax.microedition.io.file.*;
import javax.microedition.lcdui.*;




public class MediaFileSelectorImpl 
 extends List
  implements KMediaFileSelector, CommandListener, FileSystemListener
{
 // protected   Image ROOT_IMAGE;
  protected   Image UPDIR_IMAGE;
  protected   Image FOLDER_IMAGE;
  protected   Image FILE_IMAGE;

		
  protected static final String FILE_SEPARATOR = 
  	(System.getProperty("file.separator")!=null)?  System.getProperty("file.separator"): "/";
  protected static final String FILE_SEPARATOR_ALT = "/";
  
  private final static String UP_DIR = "..";

  private final Command openCommand =
    new Command("Open", Command.ITEM, 1);    
    
  private final Command cancelCommand =
    new Command("Cancel", Command.CANCEL, 1);    

  private Vector rootsList = new Vector();
  
// Stores the current root, if null we are showing all the roots
private FileConnection currentRoot = null;

protected byte[] mediaDataBlock = null;

protected String selectedFile = null;
protected String selectedURL = null;

protected KViewParent parent = null;
	
private static final boolean bDebug = false;

protected boolean bCurFolderIsARoot = true;

public MediaFileSelectorImpl()
{
	
	super("Media", List.IMPLICIT);

	if (bDebug) System.out.println("MFS building cmds....");
	
	addCommand(openCommand);
	addCommand(cancelCommand);
	setSelectCommand(openCommand);
	setCommandListener(this);
	
	try {
		if (bDebug) System.out.println("MFS load images....");

	  	//ROOT_IMAGE = Image.createImage("root_icon.png");
	  	FOLDER_IMAGE =Image.createImage("/folder_icon.png");
	 	FILE_IMAGE =Image.createImage("/file_icon.png");
	 	UPDIR_IMAGE = Image.createImage("/up_dir_icon.png");
	}
	catch (IOException imgOpenEx)
	{
		if (bDebug) System.out.println("--- icons ex: " + imgOpenEx);
	}

	
	if (bDebug) 
		{
		System.out.println("--- file sep: '" + FILE_SEPARATOR + "'");
		System.out.println("--- file sep_alt: '" + FILE_SEPARATOR_ALT + "'");
		}
	
} //constructor



/**
* @param The callback client interested in receiving finished status.
*/
public void setViewParent(KViewParent aParent)
{
	if (null == this.parent) {
		FileSystemRegistry.addFileSystemListener(this);
		resetRoots(); //display the root directories
	}
	this.parent = aParent;	
}//setViewParent
    



/**
* Cleanup any allocated resources immediately.
*/
public void doCleanup()
{
	this.selectedFile = null;
	this.selectedURL = null;
	
	if (null != currentRoot) {
		try {currentRoot.close();}
		catch (IOException ioEx) {};
		currentRoot = null;
	}
	
	mediaDataBlock = null;
	
}//doCleanup





public void commandAction(Command c, Displayable d)
{
	
	if (bDebug) System.out.println("cmd action: " + c);

	if (c == openCommand)
	{
		openSelected();
	}
	else if (c == cancelCommand)
	{
		doCleanup();
		parent.childFinished(this);

	}

}//commandAction


public void resetRoots()
{
	if (bDebug) System.out.println("resetRoots...");
	
	String defaultPhotosDir = System.getProperty("fileconn.dir.photos");
	loadRoots();
	
	if (defaultPhotosDir != null)
	{
		if (bDebug)
			System.out.println("dir.photos: " + defaultPhotosDir);
		try
		{
		  currentRoot = (FileConnection) Connector.open(  defaultPhotosDir,  Connector.READ);
		  displayCurrentRoot();
		}
		catch (Exception e)
		{
		 if (bDebug) System.out.println("### resetroot ex: " + e);
		  displayAllRoots();
		}
	}
	else
	{
		displayAllRoots();
	}
}//resetRoots




protected void displayAllRoots()
{
	if (bDebug) 
		System.out.println("displayAllRoots...");
	
	setTitle("Media");
	deleteAll();
	Enumeration roots = rootsList.elements();
	while (roots.hasMoreElements())
	{
	  String root = (String) roots.nextElement();
	  if (bDebug) 
		  System.out.println("root: " + root);
	  append(root.substring(1), FOLDER_IMAGE);
	}
	currentRoot = null;
}//displayAllRoots






protected void loadRoots()
{
	if (bDebug) System.out.println("loadRoots...");

	bCurFolderIsARoot = true;

	if (!rootsList.isEmpty())
	{
	  rootsList.removeAllElements();
	}
	
	try {
	  Enumeration roots = FileSystemRegistry.listRoots();
	  while (roots.hasMoreElements())
	  {
	    rootsList.addElement(FILE_SEPARATOR + (String) roots.nextElement());
	  }
	} catch (Throwable e) {
	  if (bDebug) 
		  System.out.println("### load roots: " + e);
	}
}//loadRoots


protected void openSelected()
{

	if (bDebug) System.out.println("openSelected....");
	int selectedIndex = getSelectedIndex();
	if (bDebug) System.out.println("selectedIndex: " + selectedIndex);
	
	if (selectedIndex >= 0)
	{
	  selectedFile = getString(selectedIndex);
	  if (bDebug) System.out.println("selectedFile: " + selectedFile);
	  
	  if (null != selectedFile)
	  {
		  if ( selectedFile.endsWith(FILE_SEPARATOR) || selectedFile.endsWith(FILE_SEPARATOR_ALT))
		  {
		    try
		    {
		      if (null == currentRoot)
		      {
	 			if (bDebug) System.out.println("new currentRoot...");
		        currentRoot = (FileConnection) Connector.open("file:///" + selectedFile, Connector.READ);
		      }
		      else
		      {
	 			if (bDebug) System.out.println("set cur root conn...");
		        currentRoot.setFileConnection(selectedFile);
		      }
		      displayCurrentRoot();
		    }
		    catch (Exception e)
		    {
		     if (bDebug) System.out.println("### open file: " + e);
		    }
		  }
		  else if (selectedFile.equals(UP_DIR))
		  {
		  	String curRootName = currentRoot.getPath()  + currentRoot.getName();
		  	if (bDebug) System.out.println("curRootName: " + curRootName);
		  	
		    if( rootsList.contains(curRootName))
		    {
		      displayAllRoots();
		    }
		    else
		    {
		      try
		      {
		        currentRoot.setFileConnection(UP_DIR);
		        displayCurrentRoot();
		      }
		      catch (IOException e)
		      {
		       	if (bDebug) System.out.println("### setfileConn: " + e);
		      }
		      catch (IllegalArgumentException e)
		      {
		      	//there's something hosed with this path-- jump back to roots
		      	displayAllRoots();
		      }
		    }
		  }
		  else
		  {
		 	if (bDebug) 
		 		System.out.println("user selected: " + selectedFile);

		  	//the user has selected a particular file
			
			
		   	//parent.childFinished(this);
		 	
		 	parent.addDeferredAction(new KMediaFileSelectorKicker(this));
		 	
		  }
		}
		else {
			if (bDebug) System.out.println("### no selected file???");
		
		}
	}
}//openSelected


protected void doNotifyOpComplete() {
	
	if (null != currentRoot)
	{
		selectedURL = currentRoot.getURL() + selectedFile;	
		if (bDebug)
			System.out.println("=== Selected URL: " + selectedURL);
		
		try {currentRoot.close();}
		catch(IOException ioEx) {}
	}
	
	currentRoot = null; //reset it
	//selectedFile = null;

	parent.childFinished(this);
}


protected void displayCurrentRoot()
{
	if (bDebug) System.out.println("displayCurrentRoot...");
	
	try
	{
	 if (null != currentRoot)
	 {
	 	String rootName = currentRoot.getName();
	 	if ((rootName == null) || (rootName.length() < 1))
	 		rootName = selectedFile;
	 		
	  setTitle("[" + rootName + "]");
	  
	 }
	 else
	 	setTitle("Media");
	 	
	  // open the root
	 deleteAll();
	 append(UP_DIR, UPDIR_IMAGE);
	 
	  // list all dirs
	  Enumeration listOfDirs = currentRoot.list("*", false);
	  while (listOfDirs.hasMoreElements())
	  {
	    String currentDir = (String) listOfDirs.nextElement();
	     
	    if (currentDir.endsWith(FILE_SEPARATOR)  || currentDir.endsWith(FILE_SEPARATOR_ALT))
	    {
	      append(currentDir, FOLDER_IMAGE);
	    }
	  }
	  // list all png files and dont show hidden files
	  Enumeration listOfFiles = currentRoot.list("*.png", false);
	  while (listOfFiles.hasMoreElements())
	  {
	    String currentFile = (String) listOfFiles.nextElement();
	    append(currentFile, FILE_IMAGE);
	  }
	  
	  listOfFiles = currentRoot.list("*.jpg", false);
	  while (listOfFiles.hasMoreElements())
	  {
	    String currentFile = (String) listOfFiles.nextElement();
		append(currentFile, FILE_IMAGE);
	  }
	  
	  
	  listOfFiles = currentRoot.list("*.gif", false);
	  while (listOfFiles.hasMoreElements())
	  {
	    String currentFile = (String) listOfFiles.nextElement();
	    append(currentFile, FILE_IMAGE);
	  }
	  
	}
	catch (Exception e)
	{
		if (bDebug) System.out.println("### displayRoot ex: " + e);
	}
	
}//displayCurrentRoot

public String getMediaName()
{
	return selectedFile;
}//getMediaName

public String getMediaMimeType()
{
	String szMimeType = null;
	
	if (null != selectedFile)
	{
		if ((selectedFile.indexOf("jpg") > 0) || (selectedFile.indexOf("jpeg") > 0))
			szMimeType = "image/jpeg";
		else if (selectedFile.indexOf("png") > 0)
			szMimeType = "image/png";
		else if (selectedFile.indexOf("gif") > 0)
			szMimeType = "image/gif";
		else if (selectedFile.indexOf("bmp") > 0)
			szMimeType = "image/bmp";
	}
	
	return szMimeType;
}//getMediaMimeType

public Image getThumbnail(int width, int height)
{
	Image thumbImage = FILE_IMAGE;
	String mediaType = getMediaMimeType();
	
	if (null != mediaType)
	{
		try {
			byte[] datablock = getMediaData(); 
			thumbImage = Image.createImage(datablock,0,datablock.length);
		}
		catch (java.lang.OutOfMemoryError oom) {
			if (bDebug) System.err.println("### OOM on createImage: "  + Runtime.getRuntime().freeMemory());
		}
		catch (Exception ex) {
			if (bDebug)  {
				System.err.println("### Couldn't create image: " + ex);
				ex.printStackTrace();
			}
			
		}
	}
	
	if (thumbImage.getWidth() > (2*width)) {
		thumbImage = FILE_IMAGE;
	}
	
	if (bDebug) System.out.println("...getThumbnail");
	
	return thumbImage;
}//getThumbnail

public byte[] getMediaData()
{
	
	if (null == mediaDataBlock)
	{
		if (null != selectedURL)
		{
			try {
			
				long availSize = 0;
									
				if (bDebug) 
					System.out.println("selectedURL: " + selectedURL);

		        currentRoot = (FileConnection) Connector.open(selectedURL);
		    	
				//currentRoot.setFileConnection(selectedFile); //relative to current directory
				
				if (bDebug) 
					System.out.println(" getting data...");
				availSize = currentRoot.fileSize();
				if (bDebug) 
					System.out.println("media file availSize: " + availSize);
				
				if (bDebug && !currentRoot.canRead())
					System.out.println("### can't read???");
				
				
				if (availSize > 0)
				{
	
					try {
						//DataInputStream is = currentRoot.openDataInputStream();
						InputStream is = currentRoot.openInputStream();
						if (bDebug) System.out.println("Creating new media block [" + availSize + "]");
		
						System.gc();
						mediaDataBlock = new byte[(int)availSize];
						if (bDebug) System.out.println("Allocated: " + availSize);
						
						is.read(mediaDataBlock);
						is.close();
						
						if (bDebug) System.out.println("...data read.");
					}
					catch (IOException ioEx)
					{
						if (bDebug) {
							System.err.println("### ioEx: " + ioEx);
							
							ioEx.printStackTrace();
						}
					}
					catch (java.lang.OutOfMemoryError oom)
					{
						if (bDebug) System.err.println("### OOM new mediaDataBlock: "  + Runtime.getRuntime().freeMemory());
					}
				}
				
				currentRoot.close(); //no longer needed
				currentRoot = null;
			}
			catch (IOException ioEx)
			{
				if (bDebug) System.out.println("### read file ioex: " + ioEx);
			}
			catch (Exception ex)
			{
				if (bDebug) System.out.println("### read file ex: " + ex);
			
			}
		}
	}
	else
	{
		if (bDebug) System.out.println("Existing mediaDataBlock [" + mediaDataBlock.length + "]");
	}
	
	return mediaDataBlock;
	
}//getMediaData


public void rootChanged(int changeType, String strArg)
{
	if (bDebug) {
		//that's nice...
		if (changeType == FileSystemListener.ROOT_ADDED) 
			System.out.println("=== FileSys: ROOT ADDED");		
		else if (changeType == FileSystemListener.ROOT_REMOVED)
			System.out.println("=== FileSys:ROOT_REMOVED");
	
		System.out.println("strArg: " + strArg);
	}
}

} //class KMediaFileSelector

class KMediaFileSelectorKicker implements Runnable {
	
	public KMediaFileSelectorKicker(MediaFileSelectorImpl aTarget) {
		
		target = aTarget;
		
	}
	
	public void run() {
		
		if (null != target) {
			
			target.doNotifyOpComplete();
		}
	}
	
	MediaFileSelectorImpl target;
}