/* Copyright (c) 2001-2005 Todd C. Stellanova, rawthought
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The  above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE. 
 * 
 * 
 *
 * 
 */

package org.kablog.midlet2;

import javax.microedition.lcdui.*;

import org.kablog.kdb.*;
import org.kablog.kgui.*;
import org.kablog.xmlrpc.KDateConverter;

import java.util.*;
import java.io.*;




/**
 *
 * @author  Todd C. Stellanova
 */
public class AdvEntryEditor extends Form 
							implements CommandListener, BlogViewChild, KViewParent, Runnable {
    
    protected EntryRecord curEntry;
    
    private ImageItem fotoItem;
    private TextField titleInput;
    private TextField bodyInput;
    private DateField creationInput;
    private ChoiceGroup postOptsInput;
    
    protected KMediaFileSelector fileSelectorView; 
    protected KCameraViewer camView; 
    protected KViewParent viewParent;
    protected BlogMgr  blogParent;
    
    protected byte[] cachedMediaData;
    protected String cachedMediaName;
    protected String cachedMediaType;

	
    private boolean	selectedChoices[];
    
    private Command cmd_Cancel; // The Done command
    private Command cmd_SaveLocal; //launch the post editor
    private Command cmd_PostNow;
    
    String propStr;
    private Command cmd_GetPhoto;
    private Command cmd_RecPhoto;
    //private Command cmd_EditPhoto;
    private Command cmd_RecAudio;
    private Command cmd_RecVideo;
    
    private static final boolean bDebug = false;
    
    
    private static final String CMDSTR_CANCEL = "Cancel";
    private static final String CMDSTR_POST_NOW = "Post";
    private static final String CMDSTR_SAVE_LOCALLY = "Save";
    
    private static final String CMDSTR_GET_PHOTO =  "Select Photo";
    private static final String CMDSTR_REC_PHOTO =  "Capture Photo";
      
    //private static final String CMDSTR_REC_VIDEO    = "Add Video";
    //private static final String CMDSTR_REC_AUDIO    = "Add Audio";  

    public static final String POST_OPTS_LIST[] = {"Allow Comments","Publish","Convert Breaks"};
    public static final boolean POST_OPTS_DEFAULTS[] = { false, true, false};

    public static final int MI_POST_OPTS_COMMENTS = 0;
    public static final int MI_POST_OPTS_PUBLISH = 1;
    public static final int MI_POST_OPTS_BREAKS = 2;

    
    Alert statusAlert;
    Object me;
    protected Thread fileSelectorThread;

  
    public AdvEntryEditor() {
        super("Edit Entry");

        curEntry = null;
        titleInput = new TextField("Title:","",64,TextField.ANY);
        bodyInput = new TextField("Body:", "<p>", 4096, TextField.ANY);
        creationInput = new DateField("Created:",DateField.DATE_TIME);
        
        //if (bDebug) System.err.println("### creating new AdvEntryEditor...");
        creationInput.setDate(new Date()); //this is the default date value...
        
         selectedChoices = new boolean[3];
         cachedMediaData = null;
         cachedMediaName = null;
         cachedMediaType = null;
         
        this.append(titleInput);
        this.append(bodyInput);
        this.append(creationInput);
        
  
        postOptsInput = new ChoiceGroup("Options:",
            ChoiceGroup.MULTIPLE, POST_OPTS_LIST, null);
		selectedChoices[0] = POST_OPTS_DEFAULTS[0];
		selectedChoices[1] = POST_OPTS_DEFAULTS[1];
		selectedChoices[2] = POST_OPTS_DEFAULTS[2];
		postOptsInput.setSelectedFlags(selectedChoices);
        this.append(postOptsInput);
 
        
        
     
     	propStr = System.getProperty("microedition.io.file.FileConnection.version");
   
        if (propStr != null) {
       		//if (bDebug) System.out.println("### fileconn vers: " + propStr);
    	    cmd_GetPhoto = new Command(CMDSTR_GET_PHOTO,Command.SCREEN,2);
            this.addCommand( cmd_GetPhoto);   
        }

        propStr = System.getProperty("video.snapshot.encodings");
	        if (propStr != null){ 
	            cmd_RecPhoto = new Command(CMDSTR_REC_PHOTO,Command.SCREEN,2);
	            this.addCommand( cmd_RecPhoto);   
	        }
	    
	        
        
        /*
        propStr = System.getProperty("supports.video.capture");
        if ((propStr != null) && (propStr.equals("true")) ){
            cmd_RecVideo = new Command(CMDSTR_REC_VIDEO,Command.SCREEN,2);
            this.addCommand(cmd_RecVideo);   
        }
        
        
        propStr = System.getProperty("supports.audio.capture");
        if ((propStr != null) && (propStr.equals("true")) ) {
            cmd_RecAudio = new Command(CMDSTR_REC_AUDIO,Command.SCREEN,2);
            this.addCommand(cmd_RecAudio);   
       }
       */

        cmd_SaveLocal = new Command(CMDSTR_SAVE_LOCALLY, Command.SCREEN, 2);
        cmd_PostNow = new Command(CMDSTR_POST_NOW,Command.SCREEN, 2);
        cmd_Cancel = new Command(CMDSTR_CANCEL, Command.SCREEN, 3);

        addCommand(cmd_PostNow);
        addCommand(cmd_SaveLocal);
        addCommand(cmd_Cancel);
        setCommandListener(this);
    }//AdvEntryEditor(KablogMIDlet myParent)
    
    /**
     * Edit the entry provided.
     */
    public AdvEntryEditor( EntryRecord entry) {
        this();
 
        if (entry != null) {
            this.curEntry = entry;
            selectedChoices[MI_POST_OPTS_COMMENTS] = entry.bAllowComments;
            selectedChoices[MI_POST_OPTS_PUBLISH] = entry.bPublish;
            selectedChoices[MI_POST_OPTS_BREAKS] = entry.bConvertBreaks;
            
            
            if (curEntry.mediaData != null) {
            	//start out with a placeholder image..
            	Image entryImg = null;
            	try { entryImg = Image.createImage("/topres/file_icon.png"); }
            	catch (IOException ioEx) {}

               if (bDebug) System.out.println("loading stored photo...");
               this.cachedMediaData = curEntry.mediaData;
               this.cachedMediaName = curEntry.mediaName;
               this.cachedMediaType = curEntry.mediaType;
               
               try {
	              Image loadImg =  Image.createImage(curEntry.mediaData,0,curEntry.mediaData.length);
	              entryImg = loadImg;
                }
                catch (java.lang.OutOfMemoryError oom)
                {
                	if (bDebug) System.err.println("### oom on load image: "  + Runtime.getRuntime().freeMemory());
                }
                
                
    			replaceFotoItem(entryImg, curEntry.mediaName);

                /*
                
                if (null != entryImg) {
                	 ImageItem imgItem = new ImageItem("Photo",entryImg,
	                        ImageItem.LAYOUT_CENTER + ImageItem.LAYOUT_NEWLINE_AFTER ,curEntry.mediaName);
	               //prepend the image item
	                this.insert(0,imgItem);
	            }
	            */
                 
    			if (bDebug) System.out.println("...Done");
            }
           
            
            if (curEntry.sTitle != null) titleInput.setString(curEntry.sTitle);
            if (curEntry.sBody != null) bodyInput.setString(curEntry.sBody);
            //override the creation date displayed, if the entry already has
            //a creation date (which it should!)
            if (curEntry.sCreationDate != null) 
                try {
                    //the following can throw a RuntimeException -- yay!
                    Date translateDate = 
                     KDateConverter.stringToDate(curEntry.sCreationDate,KDateConverter.DATE_TIME);             
                    creationInput.setDate(translateDate);
                }
                catch (Throwable t) {
                   if (bDebug) {
                    System.err.println("Bad creation date: ");
                    t.printStackTrace();
                    }
                }
        }


        if (bDebug) System.err.println("### set/get selected flags...");

	//	postOptsInput = new ChoiceGroup("Options:",
	//		ChoiceGroup.MULTIPLE, parent.POST_OPTS_LIST, null);
		postOptsInput.setSelectedFlags(selectedChoices);
		postOptsInput.getSelectedFlags(selectedChoices);
 
        
         
 
    }//AdvEntryEditor(KablogMIDlet myParent, EntryRecord entry)
    
    
    protected void replaceFotoItem(Image aImage, String aName) {
    	
        if (null != this.fotoItem) {
        	
        	for (int i = 0; i < this.size(); i++)
        	{     		
        		Item curItem = this.get(i);
        		if (curItem == fotoItem) {
        			this.delete(i);
        			break;
        		}
        	}
        	
        	fotoItem = null;
        }
        
        
        if (null != aImage) {
        	 fotoItem = new ImageItem("Photo",aImage,
                    ImageItem.LAYOUT_CENTER + ImageItem.LAYOUT_NEWLINE_AFTER , aName);
               
           //prepend the image item
            if (null != fotoItem)
            	this.insert(0,fotoItem);
        }
    }
    
    
    
     /**
     * Respond to commands, including exit
     * On the exit command, cleanup and notify that the MIDlet has been destroyed.
     */
    public void commandAction(Command c, Displayable s) {
        
        if (c == cmd_GetPhoto) {
        	this.startSelectorLaunch();
        }
       // else if (c == cmd_RecVideo) {
       //     this.doLaunchCamera();
       // }
        else if (c == cmd_RecPhoto) {
            this.doLaunchCamera();
        }
        else if (c == cmd_RecAudio) {
            //parent.doLaunchAudioRecorder
        }
        else if (c == cmd_PostNow) {            
            saveCurrentEntry(); //save any changes before attempting to send...
            
            Vector entries = new Vector(1);
            entries.addElement(new Integer(curEntry.getRecID()));
            blogParent.postEntriesAsync(entries, true);
            
            curEntry = null; //XXX no longer needed?
            System.gc();
            viewParent.childFinished(this);
            //if (bDebug) System.out.println("...done with async add.");
        }
        else if (c == cmd_SaveLocal) {
            saveCurrentEntry();
            viewParent.childFinished(this);
        }
        else if (c == cmd_Cancel) {
            //XXX should we check to see whether it has been saved already?
            //e.g. recID == invalid 
            //before losing all the data the user may have entered.
            //for now we just save the data automatically-- let the user delete it if needed
            //saveCurrentEntry();
            viewParent.childFinished(this);
        }
    }//CommandAction
    
    /**
     * Store the entry currently being edited in the entries db
     */
    protected void saveCurrentEntry() {
    	//if (bDebug) System.out.println("saveCurrentEntry...");
    	
        try {
            EntriesDB db = EntriesDB.getDB();
            if (curEntry == null)
                curEntry = db.getBlankEntry();

            try {
                //dateToString tends to throw RuntimeExceptions-- yay!
                String dateStr = 
                    KDateConverter.dateToIso8601String(creationInput.getDate(), KDateConverter.DATE_TIME);
                curEntry.sCreationDate = dateStr;
            }
            catch (Throwable t) {
            	if (bDebug) {
	                System.err.println("(save) bad sCreationDate: ");
	                t.printStackTrace();
                }
            }
            
            curEntry.sTitle = titleInput.getString();
            if ((null == curEntry.sTitle) && (null != this.cachedMediaName))
            	curEntry.sTitle = this.cachedMediaName;
            curEntry.sBody = bodyInput.getString();
            
            postOptsInput.getSelectedFlags(selectedChoices);
            curEntry.bAllowComments =	selectedChoices[MI_POST_OPTS_COMMENTS];
            curEntry.bPublish =			selectedChoices[MI_POST_OPTS_PUBLISH];
            curEntry.bConvertBreaks =	selectedChoices[MI_POST_OPTS_BREAKS];
            
            curEntry.mediaData = this.cachedMediaData;
            curEntry.mediaName = this.cachedMediaName;
            curEntry.mediaType = this.cachedMediaType;
            //curEntry.mediaURL = ""; //none to start with
            

            //save the current entry in the db
            try {db.makePersistent(curEntry);}
	        catch (java.lang.OutOfMemoryError oom)
	        {
	        	if (bDebug) System.err.println("### OOM: "  + Runtime.getRuntime().freeMemory());
				//viewParent.displayDbgMsg("Image too large-- resize or choose another image.", AlertType.ERROR);
				curEntry = null;
				
				System.gc();
	        }
        }
        catch (KDbException dbEx) {
           if (bDebug) System.err.println("Couldn't save: " + dbEx);
        }  
       
        
       // if (bDebug) System.out.println("...saveCurrentEntry");
      
    }//saveCurrentEntry
    
    
    /**
     * When the camView is done capturing an image, it calls this method.
     */
    public void childFinished(KViewChild child) {   
        
        Image thumbImg = null;
        byte[] newMediaData = null;
        
        this.cachedMediaData = null;
        this.cachedMediaName = null;
        this.cachedMediaType = null;
        System.gc();
        
        
        if (child == this.fileSelectorView) {
        
            newMediaData = fileSelectorView.getMediaData();

			if (null != newMediaData)
			{
	        	 thumbImg =  fileSelectorView.getThumbnail(320,240);//this is just a guesstimate of max size
	       	
        	    String newMediaName = fileSelectorView.getMediaName();
                if (bDebug) System.out.println("media name: " + newMediaName);
                //if we already have an image item in our form list, eliminate it
                if (this.cachedMediaData != null) {
                    this.delete(0); //image item is always at this index
                }
                this.cachedMediaData = newMediaData;
                this.cachedMediaName = newMediaName;
                this.cachedMediaType = fileSelectorView.getMediaMimeType();
			}
        	fileSelectorView.doCleanup();
        	fileSelectorView = null;
        }
        else  if (child == this.camView) {
            newMediaData = camView.getCaptureData();
            
           if (newMediaData != null) {
              thumbImg = camView.getThumbnail(120,90 );
             String newMediaName = camView.getMediaName();
               if (bDebug)  System.out.println("media name: " + newMediaName);
                //if we already have an image item in our form list, eliminate it
                if (this.cachedMediaData != null) {
                    this.delete(0); //image item is always at this index
                }
                this.cachedMediaData = newMediaData;
                this.cachedMediaName = newMediaName;
                this.cachedMediaType = camView.getMediaType();
                
           }
            else {
            	viewParent.displayDbgMsg("Bogus newMediaData??",AlertType.ERROR);
            }
            camView.doCleanup();
            camView = null;
        }
        
        
		if (thumbImg != null) {
			
			replaceFotoItem(thumbImg, this.cachedMediaName);
			
			/*
			ImageItem imgItem = new ImageItem(null,thumbImg,
			    ImageItem.LAYOUT_CENTER + ImageItem.LAYOUT_NEWLINE_AFTER 
			     , this.cachedMediaName);
			this.insert(0,imgItem);
			*/
			
			String curTitle = titleInput.getString();
			if ((null == curTitle) || (curTitle.length() < 1))
            	titleInput.setString(this.cachedMediaName);
 
			saveCurrentEntry();
		}
		
    	viewParent.reqSetVisible(this);
    }
    
    
    protected void startSelectorLaunch() {
    	System.out.println("startSelectorLaunch...");
    	
    	fileSelectorThread = new Thread(this);
    	fileSelectorThread.start();
    	//viewParent.addDeferredAction(this);
    }
       
   //From Runnable interface
    
    public void run() {
    	doLaunchSelector();
    }
    
   
    
    public void doLaunchCamera() {
        
        //If the platform supports MMAPI (e.g. on a Nokia3650 or n-gage?)
        //and it supports video.encodings
        //Then attempt to launch the CameraDisplay
        //If not (e.g. on the emulator), fake it!
        String sVidEncodings = System.getProperty("video.snapshot.encodings");
        
        if ( sVidEncodings != null)  
        {
            try {
                    try {
                        camView =  CameraDisplayFactory.getInstance();  
                    }
                    catch (Exception instExt) {
                    	System.out.println("new camView ex: " + instExt);
                        viewParent.displayDbgMsg("new CamForm threw: " + instExt, AlertType.ERROR);
                    }
                    
                    if (camView != null) {
                        camView.setViewParent(this);
                        viewParent.reqSetVisible((Displayable)camView);  
                        //((CameraDisplay)camView).initCamera(160,120);
                    }

            }
            catch (Exception ex) {
            	System.out.println("doLaunchCam ex: " + ex);
                viewParent.displayDbgMsg("Couldn't instantiate cls: \n" + ex.toString(), AlertType.ERROR);
            }     
        }
        /*
        else {
        //Fake camView
            camView = new FakeCamDisplay();
            camView.setViewParent(this);
            viewParent.reqSetVisible((FakeCamDisplay)camView); 
            ((FakeCamDisplay)camView).initCamera(160,120);       
        }
        */
    }
  
    
    public void doLaunchSelector() {
        
    	System.out.println("doLaunchSelector...");
    	
    	fileSelectorView = null;
    	
    	String propStr = System.getProperty("microedition.io.file.FileConnection.version");
    	   
        if (propStr != null)
        {
	    	try {
	    		fileSelectorView = MediaFileSelectorFactory.getInstance();
	    		fileSelectorView.setViewParent(this);
	    		viewParent.reqSetVisible((Displayable)fileSelectorView); 
	    	}
	    	catch (Exception ex)
	    	{
	    		if (bDebug) System.out.println("### selector fail: " + ex);
	    	}
    	}
        	
        
    	
    }//doLaunchSelector
    
    
   /** We've updated the child's status.
     */
    public void childStatusChanged(KViewChild child, int statusType, int status) {
        if (bDebug) System.out.println("Child status changed: " + status);
    } 
    
    /** @param newView object o make visible, if possible.
     */
    public void reqSetVisible(Displayable newView) {
        viewParent.reqSetVisible(newView);
    }
    
    /** @param parent to set
     */
    public void setBlogParent(BlogMgr parent) {
        this.blogParent = parent;
    }
    
    /** @param The callback client interested in receiving finished status.
     */
    public void setViewParent(KViewParent parent) {
        this.viewParent = parent;
    }
    
    /** 
     Display a debug message somehow
     */
    public void displayDbgMsg(String msg, AlertType type) {
         if ((this.bDebug) && (this.curEntry != null)) {
             
             if (bDebug) System.out.println("dbgMsg: " + msg);
             try {
               if (this.curEntry != null) {
                     this.curEntry.sBody += msg;
                 }
                 int curPos = bodyInput.getCaretPosition();
                 this.bodyInput.insert(msg,curPos);    
               }
             catch (Exception ex) {
                 viewParent.displayDbgMsg(msg,type);
             }
         }
    }
    
    public void doCleanup()
    {
    	if (bDebug) System.out.println("AdvEntryEdtor.doCleanup...");
    	this.curEntry = null;
    	this.cachedMediaData = null;
    	this.cachedMediaName = null;
    	this.cachedMediaType = null;
    	System.gc();
    }
    
    public void addDeferredAction(Runnable runny)
    {
    	viewParent.addDeferredAction(runny);
    }
//doLaunchCacheViewer
       
}//class AdvEntryEditor
