/* Copyright (c) 2001-2005 Todd C. Stellanova, rawthought
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The  above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE. 
 * 
 * 
 * Created on October 31, 2002, 12:34 PM
 */
package org.kablog.midlet2;

import javax.microedition.lcdui.*;
import java.util.*;
import java.io.*;
import javax.microedition.io.*;

/**
 * This class allows the user to select which Blog they're interested in 
 * working with. Isn't that nice?
 *
 * @author  Todd C. Stellanova
 */
public class BlogSelectorForm extends SetupFormBase {
    
    protected static final boolean bDebug = false;
    private ChoiceGroup selectionInput;
    private Vector blogsList;
    private String sCurBlogID = null;

    public BlogSelectorForm(KablogMIDlet myParent, Vector blogs) {
      super("Select Blog",myParent);   
     //System.out.println("BlogSelectorForm blogs: " + blogs);
     blogsList = blogs;
     addTrueFormItems();
     
    }//constructor
    
    protected void addTrueFormItems() {
        try {
          if (blogsList != null) {
             int numBlogs = blogsList.size();
             int curSelected = -1;
             String[] blogNames = new String[numBlogs];
             for (int i = 0; i < numBlogs; i++) {
                 Hashtable blogData = (Hashtable)blogsList.elementAt(i);   
                 blogNames[i] = (String)blogData.get("blogName");
                 if (this.sCurBlogID != null)  {
                     String blogID =(String)blogData.get("blogid");
                     if (blogID.equals(this.sCurBlogID))
                        curSelected = i;
                 }
             }
             //XXX add image icons to the choice group?
             selectionInput = new ChoiceGroup("Target Blog:",Choice.EXCLUSIVE,blogNames, null);
             
             if (curSelected != -1) {
                 selectionInput.setSelectedIndex(curSelected,true);
             }
             append(selectionInput);
         }
         else {
             //System.err.println("blogsList null?");
             append(new StringItem("-Error-","Couldn't load blogs list."));
             removeCommand(cmd_Save); //nothing to save
         }
     }
      catch (Exception ex) {
        if (bDebug) System.err.println("build blogsList ex: " + ex);
        append(new StringItem("-Error-","Couldn't build blogs list."));
        removeCommand(cmd_Save); //nothing to save
      }      
        
    }//addTrueFormItems
    
    public void addFormItems(KablogPrefs prefs) {
         
        if (prefs.bBlogSelected)
            this.sCurBlogID = prefs.sCurBlogID;
        
        if (blogsList != null) {
            addTrueFormItems();
        }

    }//addFormItems
    
    
    public void collectFormData(KablogPrefs prefs) {
        int selection = selectionInput.getSelectedIndex();
        if (selection >= 0) {
            Hashtable blogData = (Hashtable)blogsList.elementAt(selection);   
            String blogID = (String)blogData.get("blogid");
            if (blogID != null) {
                prefs.sCurBlogID = blogID;
                prefs.bBlogSelected = true;
            }
        }
    }//collectFormData
    
//CommandAction
    
}//class UserSetupForm

