
/* Copyright (c) 2001-2005 Todd C. Stellanova, rawthought
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The  above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE. 
 * 
 * 
 * Created on November 7, 2002, 6:49 PM
 */

package org.kablog.midlet2;

import javax.microedition.lcdui.*;

import org.kablog.kdb.*;
import org.kablog.kgui.*;

import java.util.*;


/**
 * This class displays a list of blog entries in the entry cache.
 * The user can select one or more blog entries to post, edit, or delete.
 *
 * @author  Todd C. Stellanova
 */
public class CacheViewer extends MultiEntryPicker 
	implements ItemStateListener {
   
   protected static final boolean bDebug = false;
   
    Command cmd_PostSelected;
    Command cmd_EditFirstSelected;
    Command cmd_DeleteSelected;
  
    /** Creates a new instance of CacheViewer */
    public CacheViewer(EntriesDB myDB, KViewParent myParent) {
        super("Cached Entries",myDB,myParent);
        
        cmd_EditFirstSelected = new Command("Edit",Command.SCREEN,2);
        cmd_PostSelected = new Command("Post",Command.SCREEN,2);
        cmd_DeleteSelected = new Command("Delete",Command.SCREEN,3);
        
   
        //if (debugOn) System.out.println("Added all commands");
        this.setCommandListener(this);
        this.setItemStateListener(this);
    }//CacheViewer
    
    
    /**
     * Handle selection changes to the list
     */
    
    
    public void itemStateChanged(Item item) {
        if (choices.equals(item)) {
            selectedFlags = new boolean[choices.size()];
            //System.out.println("### total choices: " + selectedFlags.length);
            choices.getSelectedFlags(selectedFlags);
            
            //now, count how many indices are selected...pain in my ass!
           int selectedCount = 0;
            for (int i = 0; i < selectedFlags.length; i++) {
                if (selectedFlags[i]) selectedCount++;
            }
            
            //System.out.println("### selectedCount: " + selectedCount);
            
            if (selectedCount > 0) {
                if (selectedCount == 1)
                    addCommand(cmd_EditFirstSelected);
                else
                    removeCommand(cmd_EditFirstSelected);//has no effect if it's not added already

                addCommand(cmd_PostSelected);
                addCommand(cmd_DeleteSelected);                  
            }
            else {
                removeCommand(cmd_EditFirstSelected);//has no effect if it's not added already
                removeCommand(cmd_PostSelected);
                removeCommand(cmd_DeleteSelected);                  
            }
        }
    }//ItemStateChanged
    
    
    
    /**
     * Handle Command events sent by the gui.
     * This method recursively calls its super's method-- is that necessary?
     */
    public void commandAction(Command c, Displayable d) {
        
        //if (debugOn) System.out.println("CacheViewer cmdAction: " + c);
        
        if (c == cmd_PostSelected) {
            //post ALL of the selected items
            Vector selectedEntries = getSelectedEntries();
            blogParent.postEntriesAsync(selectedEntries,true);
            blogParent.childFinished(this);
        }
        else if (c == cmd_EditFirstSelected) {
            //load that item as the currently targeted blog entry
            //load into base midlet and tell midlet to edit
            selectedFlags = new boolean[choices.size()];
            int numSelected = choices.getSelectedFlags(selectedFlags);
           //  if (debugOn) System.out.println("numSelected: " +numSelected);
            if (numSelected > 0) { //TODO need at least one..select the first one
                for (int i = 0; i < selectedFlags.length; i++) {
                    if (selectedFlags[i]) {
                        String key = choices.getString(i);
                        Integer recID = (Integer)entriesHash.get(key); //TODO store whole entry instead?
                        System.out.println("### recID: " + recID);
                        try {
                            EntryRecord target = (EntryRecord)db.getEntryByID(recID.intValue());
                            blogParent.doEditEntry(target);
                        }
                        catch (KDbException dbEx) {
                           if (bDebug) System.err.println("getEntryByID dbEx: " + dbEx);
                        }
                    }
                }
                
            }
            else {
                //TODO tell the user you can't edit entries you haven't selected?
            }
        }
        else if (c == cmd_DeleteSelected) {
            //TODO Maybe need some kind of confirm "Are you sure you want to delete these items?"
            //delete ALL of the selected items
            Vector selectedEntries = getSelectedEntries();
            for (int i = 0; i < selectedEntries.size(); i++) {
                try {
                    Integer recID = (Integer)selectedEntries.elementAt(i);
                    db.removeEntryByID(recID.intValue());
                    removeEntryFromDisplayList(recID);
                }
                catch (KDbException dbEx) {
                   if (bDebug) System.err.println("Delete rec dbEx: " + dbEx);
                }
            }
            viewParent.childFinished(this);
        }
        else 
            super.commandAction(c,d);//call this to handle Exit events
        
    }
  
    
//commandAction
    

    
}//class CacheViewer
