
/* Copyright (c) 2001-2005 Todd C. Stellanova, rawthought
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The  above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE. 
 * 
 *
 * Created on November 7, 2002, 4:54 PM
 */

package org.kablog.midlet2;

import java.util.*;
import java.io.*;

import org.kablog.kdb.*;



/**
 * Stores all supported Blog entry fields.
 *
 * To do: add support for Categories (multiple)
 *
 * @author  Todd C. Stellanova
 */
public class EntryRecord extends KDbRecord {
    
    public String sCreationDate; //date entry was created, in an encoded format familiar to MT
 
    public boolean bAllowComments = false; //allow comments?
    public boolean bPublish = true; //to publish or not to publish...or perish?
    public boolean bConvertBreaks = false; //convert line breaks into paragraph tags?

    public String sTitle ="";
    public String sBody = "";
    
    public byte[]   mediaData = null; //raw media data
    public String	mediaType = "image/jpeg"; //MIME type of media
    public String   mediaName = "untitled media"; //for internal use
    public String   mediaURL = ""; //provided by the server
    
    
    //public Vector vCategories;
    
	/*
    public String toString() {
        return "Entry: " + recID+
             "\nCreated: " +sCreationDate+
             "\nPublish: " +bPublish+
             "\nConv Brks: " +bConvertBreaks+
             "\nComments: " +bAllowComments+
             "\nTitle: " +sTitle+
             "\nBody: " +sBody+
             "\n";
            
    }//toString
	*/
    
    
    public int guessRecordSize() {
		int totalSize = 0;
		
		totalSize += this.sCreationDate.length();
		
		totalSize += 8; //for booleans
		
		totalSize += this.sTitle.length();
		totalSize += this.sBody.length();

       if ((this.mediaData != null) && (this.mediaData.length > 0)) {
       
       	totalSize += this.mediaData.length;
       	totalSize += this.mediaType.length()
       	
       	;
       	totalSize += this.mediaName.length();
       	if (null != mediaURL) totalSize += this.mediaURL.length();
       }
       
       totalSize += 24; //for good measure
 
 		return totalSize;
	}//guessRecordSize
	
    /**
     * Write this record's data to the stream provided. (similar to serializing)
     */
    public void writeToStream(DataOutputStream os) throws IOException {
        //must give superclass a chance to write out a header
        super.writeToStream(os);
		
        os.writeUTF(sCreationDate);

       os.writeBoolean(this.bAllowComments);
        os.writeBoolean(this.bPublish);
        os.writeBoolean(this.bConvertBreaks);

         os.writeUTF(sTitle);
        os.writeUTF(this.sBody);
       
        if ((this.mediaData != null) && (this.mediaData.length > 0)) {
            //System.out.println("### mediaData.len: " + mediaData.length);
            os.writeInt(this.mediaData.length);
            os.write(this.mediaData);
            //System.out.println("### mediaType: " + mediaType);
            os.writeUTF(this.mediaType);
            //System.out.println("### mediaName: " + mediaName);
            os.writeUTF(this.mediaName);
            // System.out.println("### mediaURL: " + mediaURL);
           os.writeUTF(this.mediaURL);
        }
        else
            os.writeInt(0); //indicate no mediaData

    }//writeToStream
    
    /**
     * Read this record's data from an input stream.
     */
    public void readFromStream(DataInputStream is) throws IOException {
        //must give superclass a chance to read header
        super.readFromStream(is);

        this.sCreationDate = is.readUTF();

        this.bAllowComments = is.readBoolean();
        this.bPublish = is.readBoolean();
        this.bConvertBreaks = is.readBoolean();

        this.sTitle = is.readUTF();
        this.sBody = is.readUTF();

        int varDataLen = is.readInt();
        if (varDataLen > 0) {
            this.mediaData = new byte[varDataLen];
            //System.out.println("### varDataLen: " + varDataLen);
           if (mediaData != null) {
                is.read(this.mediaData);
                this.mediaType = is.readUTF();
                //System.out.println("### mediaType: " + mediaType);
                this.mediaName = is.readUTF();
                //System.out.println("### mediaName: " + mediaName);
                this.mediaURL = is.readUTF();
                //System.out.println("### mediaURL: " + mediaURL);
            }
        }
        else
            this.mediaData = null;
        
       // System.out.println("### readFromStream complete");
    }//readFromStream
    
}//class EntryRecord
