
/* Copyright (c) 2001-2005 Todd C. Stellanova, rawthought
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The  above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE. 
 * 
 * 
 * Created on May 26, 2003, 6:35 PM
 */

package org.kablog.midlet2;


import javax.microedition.lcdui.*;

import org.kablog.kdb.*;
import org.kablog.kgui.*;

import java.util.*;
import java.io.*;


/**
 * @author  Todd C. Stellanova
 */
public class FakeCamDisplay extends BaseCamDisplay {
    
    protected static final boolean bDebug = false;
    InputStream imageStream;
    public static final String JPEG_RES_FILENAME = "res/jpeg-testimg.jpg";
    public static final String PNG_RES_FILENAME = "res/png-testimg.png";
        
    
   
    
    protected byte[] getImageDataFromFile() {
        byte[] rawData = null; // ~ 32kB buffer
        
        try {
           if (bDebug)  System.out.println("### Loading: " + JPEG_RES_FILENAME);
            imageStream = this.getClass().getResourceAsStream(JPEG_RES_FILENAME);
            if (imageStream != null) {
                int count = imageStream.available(); //how big is the image data?
                if (bDebug) System.out.println("### available: " + count);
                if (count == 0) count = 32767;
                rawData = new byte[count]; // ~ 32kB buffer
                count = imageStream.read(rawData);
                if (bDebug) System.out.println("### read: " + count);
                if (count <= 0)
                    rawData = null;
                else {
                    //compact the data
                    byte[] compactData = new byte[count];
                    if (compactData != null) {
                        System.arraycopy(rawData,0,compactData,0,count);
                        rawData = compactData;
                        if (bDebug) System.out.println("### compact len: " + compactData.length);
                    }
                }
                
            }
            else
                if (bDebug) System.err.println("### no image stream");
        }
        catch (IOException ioEx) {
             if (bDebug) System.err.println("getRawSnapshotData ioEx: " + ioEx);
        }
        
        return rawData;
    } //getImageDataFromFile
    
    
    
    protected void doRecordPhoto() {
        try {
            this.curMediaData = this.getImageDataFromFile();
            
             //player.stop(); // "pause" the display of video data...
            if (this.curMediaData != null) {                
                if (bDebug) System.out.println("### Creating image from jpeg...");
                snapShotImage = Image.createImage(this.curMediaData,0,this.curMediaData.length);
                this.curMediaName = JPEG_RES_FILENAME;
                if (bDebug) System.out.println("### Create image done!");                 
            }
            else {
                if (bDebug) System.out.println("### Loading: " + PNG_RES_FILENAME);
                snapShotImage = Image.createImage(PNG_RES_FILENAME);
                this.curMediaName = PNG_RES_FILENAME;
            }
        }
        catch (IOException ioEx) {
             if (bDebug) System.err.println("### doRecordPhoto ioEx: " + ioEx);
        }
        catch (Exception ex) {
             if (bDebug) System.err.println("### doRecordPhoto ex: " + ex);
        }
        
    }
    
    
        protected void paint(Graphics gw) {
        
        
        switch (curState) {
                  
            case STATE_PHOTO_VIEWFINDER:
                if (this.snapShotImage != null) {
                    gw.drawImage(this.snapShotImage,gw.getClipWidth()/2,gw.getClipHeight()/2,
                        Graphics.HCENTER|Graphics.VCENTER);
                }   
                else {
                     //paint a "please wait" ?
                    gw.drawString("Initializing...",gw.getClipWidth()/2,gw.getClipHeight()/2,
                        Graphics.HCENTER|Graphics.VCENTER);    
                }
              
              break;
                
                
          default:
              super.paint(gw);
                break;         
        } //witch curstate
   }//paint
    
    
     
    protected void doDisplayViewFinder() {
        doRecordPhoto(); //first, load the static image data from res file
    }
    
    public void doCleanup() {
        this.curMediaData = null;
        this.curMediaName = null;
        this.snapShotImage = null;
    }
    
    
}//class CameraDisplay
