/* Copyright (c) 2001-2005 Todd C. Stellanova, rawthought
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The  above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE. 
 * 
 * 
 * Created on October 27, 2002, 5:09 PM
 */

package org.kablog.midlet2;




import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;

import org.kablog.kdb.*;
import org.kablog.kgui.*;
import org.kablog.xmlrpc.KXmlRpcFault;

import java.util.*;
import java.io.*;

//import javax.microedition.media.*;


/**
 *
 * @author  Todd C. Stellanova
 */
public class KablogMIDlet extends MIDlet implements BlogMgr, Runnable  { 
    private Screen curScreen;
    private Displayable curDisplayable;
    protected KablogEngine engine;
    Display display;    // The display for this MIDlet
    private MainMenu menu;
    private boolean bAlreadyRunning = false;
    private static final boolean bDebug = false;
    
    protected Screen statusDialog;
    protected int asyncActionType;
    protected Thread engineThread;
    
   // private static final String REG_OK_TITLE = "Registered OK";
   // private static final String REG_OK_BODY = "Thank you for registering! Your contribution helps support improvements.";
        
  //  private static final String REG_FAIL_TITLE = "Invalid Code";
  //  private static final String REG_FAIL_BODY = "See www.kablog.org for instructions.";
    


    public KablogMIDlet() {
        display = Display.getDisplay(this);     
        //doSplashScreen();
        menu = new MainMenu(this);
        display.setCurrent(menu);

        display.callSerially(this);
     }
    
    /**
     * Clear all the screens that we've opened.
     */
    protected void clearScreens() {
       menu = null;
       curScreen = null;
       statusDialog = null;
       curDisplayable = null;
    }//clearScreens
    
    protected void doMainMenu() {
        clearScreens();
        menu = new MainMenu(this);
        display.setCurrent(menu);

    }//doMainMenu
    
    public void childFinished(KViewChild child) {
        if (statusDialog == null)
            doMainMenu();
    }
    
    public void doLaunchCacheViewer() {
        clearScreens();
        EntriesDB db = EntriesDB.getDB();
        CacheViewer cacheView = new CacheViewer(db, this);
        cacheView.setBlogParent(this);
        cacheView.setViewParent(this);
        curScreen = cacheView;
        display.setCurrent(curScreen);
    }//doLaunchCacheViewer
    
 
    public void postMediaData(byte[] mediaData, String mediaType, String mediaName) {
        Object result = "Failed";

        try {
         result =
            engine.postMediaObject(mediaData,mediaType,mediaName);
        }
        catch (Exception ioEx) {
            
        }
        clearScreens();
        if (!(result instanceof String))
            result = "Couldn't post media object.";
        
        Alert resAlert = new Alert("newMediaObject", (String)result, null, AlertType.WARNING) ;
        resAlert.setTimeout(Alert.FOREVER);
        display.setCurrent(resAlert,this.curDisplayable);
        //System.out.println("result: " + result);
    }
    /**
     * @param List of Integer entry IDs
     */
    public void postEntriesAsync(Vector entryIDs, boolean deleteWhenDone) {
        Vector parms = new Vector(2);
        parms.addElement(entryIDs);
        parms.addElement(new Boolean(deleteWhenDone)); //?delete from cache when finished
        asyncActionType = KablogEngine.POST_MULT;
        engine.setupAsyncOp(asyncActionType,parms);
        engineThread = new Thread(engine);
        showActionStatus(KablogEngine.POST_MULT);
        engineThread.start();
    }//postEntriesAsync
    
    /**
     * Launch the Blog Selector prefs form
     */
    public void doTargetBlogSelect() {
        clearScreens();
        asyncActionType = KablogEngine.GET_BLOGSLIST;
        engine.setupAsyncOp(asyncActionType,null);
        engineThread = new Thread(engine);
        showActionStatus(KablogEngine.GET_BLOGSLIST);
        engineThread.start();
    }//doTargetBlogSelect
    
    
    /**
     * Tell the user what's going on, while we process asynchronous actions.
     */
    protected void showActionStatus(int type) {
        String statusTitle = "Bogus", statusMsg = "Bogus";
        boolean bAllowCancel = false;
        this.asyncActionType = type;
        
        switch (type) {
            case (KablogEngine.POST_MULT):
                statusTitle = "Posting";
                statusMsg = "Please wait.";
                break;
            case (KablogEngine.GET_BLOGSLIST):
                statusTitle ="Retrieving...";
                statusMsg = "Retrieving list of blogs. Please wait.";
                break;
        }

        System.out.println("Setting up new StatusDialog...");
        statusDialog = new StatusDialog(statusTitle,statusMsg,engine,this, bAllowCancel);
        System.out.println("calling display.setCurrent...");
        display.setCurrent(statusDialog);
    }//showActionStatus
    
    /**
     * The asynchronous action we started has completed.
     */
    public void asyncOpComplete(Object asyncResult, int lastErr) {
        
        //System.out.println("### asyncOpComplete: " + result + " lastErr: " + lastErr);
        
        clearScreens();
        
        if (lastErr != 0) {

            this.menu = new MainMenu(this);
            String failDetails ;
            switch (lastErr) {
                case KProgressStatus.ERR_BAD_USER:
                    failDetails = "Bad username or password.";
                    break;
               case KXmlRpcFault.BAD_TARGET:
                    failDetails = "Invalid blog ID. Reselect target blog.";
                    break;
                case KProgressStatus.ERR_IO:
                    failDetails = "Could not connect to server.";
                    break;
                case KXmlRpcFault.USER_HAS_NO_BLOGS:
                    failDetails = "User has no blogs. You may need to create some.";
                    break;
                     
                default:
                    if (asyncResult instanceof KXmlRpcFault) {
                        failDetails = ((KXmlRpcFault)asyncResult).getMessage();
                    }
                    else
                        failDetails = "Error code: " + lastErr;
                    break;
                
            }

            Alert failureAlert = new Alert("Failed",failDetails,null,AlertType.ERROR);
            failureAlert.setTimeout(Alert.FOREVER);
            display.setCurrent(failureAlert,menu);
   
       } //if lastErr != 0   
       else {
            
            switch (asyncActionType) {
                case (KablogEngine.POST_MULT):
                    //System.out.println("POST_MULT complete...doMainMenu");
                    doMainMenu();
                    break;

                case (KablogEngine.GET_BLOGSLIST):
                    curScreen = new BlogSelectorForm(this, (Vector)asyncResult);   
                    display.setCurrent(curScreen);
                    break;

                default:
                    System.err.println("unk asyncOp: " + asyncActionType);
            }
        }
        
        asyncActionType = 0;
        engineThread = null;
        
    }//asyncOpComplete
    
    /**
     * Edit the entry provided (using the Adv Entry form)
     */
    public void doEditEntry(EntryRecord entry) {
        clearScreens();
        //System.out.println("### creating new adventryeditor");
        AdvEntryEditor editore = new AdvEntryEditor(entry);
        editore.setViewParent(this);
        editore.setBlogParent(this);
        curScreen = editore;
        display.setCurrent(curScreen); 
    }//doEditEntry
    
    /**
     * Launch the blog entry editor.
     */
    public void doLaunchAdvEditor() {
        doEditEntry(null);   
    }//doLaunchAdvEditor

    
    /**
     * Display registration form
     */
    /*
    public void doRegister() {
        clearScreens();
      
        KablogPrefsDB db = KablogPrefsDB.getDB();
        KablogPrefs prefs = db.getKablogPrefsRecord();
 
        
        if (!prefs.bChalGenerated) {
            prefs.sRegChal = RegMgr.genChalStr();
            prefs.installDate = System.currentTimeMillis();
            prefs.bChalGenerated = true;
            db.flushPrefs(); //write out the new chal str
        }  
                
       
        curScreen = new RegForm(this,prefs);
        display.setCurrent(curScreen);
    }//doRegister
    */

    
    
    /**
     * Verify the registration code from the server
     */
    /*
    public void doRegVerify(KablogPrefs prefs, String answer) {
        Alert statusDisplay;
       boolean bConfirmed = RegMgr.regIsValid(prefs.sRegChal,answer);
        
        if (bConfirmed ) {          
            //write the confirmation code to the prefs db
            prefs.sRegResp = answer;
            prefs.bRespReceived = true;
            KablogPrefsDB db = KablogPrefsDB.getDB();
            db.flushPrefs();
            
            clearScreens();
            menu = new MainMenu(this);  
            statusDisplay = new Alert(REG_OK_TITLE,REG_OK_BODY,
                 null, AlertType.CONFIRMATION);
            statusDisplay.setTimeout(Alert.FOREVER);
            display.setCurrent(statusDisplay,menu);
        }
        else {
            statusDisplay = new Alert(REG_FAIL_TITLE,
            REG_FAIL_BODY,null,AlertType.ERROR);
            statusDisplay.setTimeout(Alert.FOREVER);
            display.setCurrent(statusDisplay);
        }
    }//doRegVerify
    */
    
    /**
     * Launch the User Setup prefs form
     */
    public void doUserPrefsEdit() {
        clearScreens();
        curScreen = new UserSetupForm(this);
        display.setCurrent(curScreen);
    }
    
    /**
     * Launch the Host Setup prefs form
     */
    public void doHostPrefsEdit() {
        clearScreens();
        curScreen = new HostSetupForm(this);
        display.setCurrent(curScreen);
    }
    

    
    public void doExitApp() {
        destroyApp(false);
        notifyDestroyed();
    }

    /**
    protected void doSplashScreen() {
            Image splashImg = null;
            String imgName = "bogus";
            
 
                try {
                    imgName = "/topres/k20-icon.png";
                    splashImg = Image.createImage(imgName);
                }
                catch (IOException ioEx) {
                   // System.out.println("### Couldn't create splash image: " + imgName);
                   imgName = null;
                }
                finally {
                    //System.out.println("### Creating splashAlert: " + imgName + " : " + splashImg);
                    //now that displayable is current...create main menu and display asap
                    menu = new MainMenu(this);
                    display.setCurrent(menu);
                }

    }
    */
    
    /**
     * Start the application.
     * This is where we show the splashscreen
     */
    public void startApp() {
 
        
        if (!this.bAlreadyRunning) {
                       bAlreadyRunning = true;

         } //if ! bAlreadyRunning

            
    }//startApp
    
    /**
     * Pause is a no-op since there are no background activities or
     * record stores that need to be closed.
     */
    public void pauseApp() {
        //XXX close record stores?
        //System.out.println("pauseApp unsupported.");
    }
    
    /**
     * Destroy must cleanup everything not handled by the garbage collector.
     */
    public void destroyApp(boolean unconditional) {
        if (!unconditional) {
           //XXX ask the user if they wish to save any unsaved data?
        }
            
        clearScreens();
        //XXX cleanup engine?
        engine = null;
        
    }//destroyApp
    
    
    
    
    //For testing profiles
    
    String allProps;
    
    
    void doProfileTest()
    {
        testDeviceProfile();
        
        if (!this.bDebug) {
            Alert profileAlert = new Alert("Props",allProps,null,AlertType.INFO);
            profileAlert.setTimeout(Alert.FOREVER);
            display.setCurrent(profileAlert,menu);//return to main menu when finished
        }

    }//doProfileTest
    
    void checkProperty(String propName)
    {
        String tmpValStr = System.getProperty(propName);
        if (tmpValStr != null) {
            allProps +=  propName + ": " + tmpValStr + "\n";
            //System.out.println(propName + ": " + tmpValStr);           
        }
    }
    void testDeviceProfile()
    {
          
        //allProps = "Props. supported: \n";
        allProps = "";
        
        checkProperty("video.capture.encodings");
        checkProperty("video.encodings");
        checkProperty("video.snapshot.encodings");

        /*
        checkProperty("microedition.profiles");
        checkProperty("m.SKT_API");
        checkProperty("microedition.platform");
        checkProperty("microedition.encoding");
        checkProperty("microedition.locale");
        checkProperty("microedition.profile");
        checkProperty("microedition.http_proxy");
        checkProperty("profiles.imei");
        checkProperty("phone.imei");
        checkProperty("phone.mcc");
        checkProperty("phone.mnc");
        checkProperty("phone.lai");
        checkProperty("phone.cid");
        checkProperty("supports.recording");
        checkProperty("supports.video.capture");
        checkProperty("supports.audio.capture");
        checkProperty("audio.encodings");
        checkProperty("audio.snapshot.encodings");
        checkProperty("video.capture");
        
        checkProperty("supports.mixing");
        checkProperty("phone");
        checkProperty("java.version");
        checkProperty("com.sun.midp.inethttp");

        checkProperty("kddi.system.wakeupMode");
        checkProperty("nokia.system");        
         */
       /*
        try {
            String prot = "capture://video";
            
            String ptest[] = Manager.getSupportedContentTypes(prot);
            if (ptest != null) {
                for (int i= 0; i< ptest.length; i++) {
                    String ctype = ptest[i];
                    allProps +=  ">>>"+ prot + ": " + ptest[i] + "\n";
                    
                   String ctest[] = Manager.getSupportedProtocols(ctype);
                    if (ctest != null) {
                        for (int j= 0; j< ctest.length; j++) {
                            String ptype = ctest[j];
                            allProps +=  ">" + ctype + ": " + ptype + "\n";
                        }
                    }

                }
            }
        }
        catch (Exception ex) {
            allProps += "<<< " + ex + "\n";
        }
        **/
        
        displayDbgMsg(allProps, AlertType.INFO);

     }
    
    /** We've updated the child's status.
     */
    public void childStatusChanged(KViewChild child, int statusType, int status) {
        if (bDebug) System.out.println("Child status changed: " + status);
    }    
 
    /** @param newView object o make visible, if possible.
     */
    public void reqSetVisible(Displayable newView) {
        this.display.setCurrent(newView);
    }    
 
    public void displayDbgMsg(String msg, AlertType type) {
           if (bDebug) System.err.println("### msg: " + msg);
            
     
            //Alert statusAlert = new Alert("Error", msg, null, type);
            //statusAlert.setTimeout(Alert.FOREVER);
            //display.setCurrent(statusAlert);
           
            /*
            if (null != this.curDisplayable)
                display.setCurrent(statusAlert,this.curDisplayable);
            else {
                this.curDisplayable = statusAlert;
                display.setCurrent(statusAlert);
            }
            */
            
            
    }   //displayDbgMsg 
    
    public void run() {
        
        //for deferred setup stuff
        if (this.engine == null) {
            engine = new KablogEngine();
        }
    }
    
    
    public void addDeferredAction(Runnable runny)
    {
    	display.getDisplay(this).callSerially(runny);
    	
    }

    
}//class kablogMIDlet









