/* Copyright (c) 2001-2005 Todd C. Stellanova, rawthought
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The  above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE. 
 * 
 * 
 * Created on October 28, 2002, 2:12 PM
 */
package org.kablog.midlet2;

import javax.microedition.rms.*;

import org.kablog.kdb.*;
import org.kablog.kprefs.*;

import java.io.*;

/**
 * Class used exclusively to store and retrieve user application prefs.
 * @author  Todd C. Stellanova
 */
public class KablogPrefs extends KDbRecord {
    public static final int     SERVTYPE_MOV_TYP = 0;
    public static final int     SERVTYPE_METAWEB = 1;
    public static final int     SERVTYPE_BLOGGER = 2;
    
    public String sURL;
    public boolean bHostInfoEdited;
    public int      nBlogServerType;
    public String sUsername;
    public String sPassword;
    public boolean bUserInfoEdited;
    public String sCurBlogID;
    public boolean bBlogSelected;
    
    public boolean bChalGenerated;
    public boolean bRespReceived;
    public long   installDate;
    public boolean bDemoExpired;
    public String  sRegChal;
    public String  sRegResp;
    
    public KablogPrefs() {
           //this.sURL = "http://www.mysite.com/mt/mt-xmlrpc.cgi";
           this.sURL = "http://www.kablog.org/cgi-bin/mt/mt-xmlrpc.cgi";
           this.sUsername = "buddy";
           this.sPassword = "asap5";
           this.sCurBlogID = "2";
           this.bBlogSelected = false;
           this.bHostInfoEdited = false;
           this.nBlogServerType = SERVTYPE_MOV_TYP;
           this.bUserInfoEdited = false;
           this.bChalGenerated = false;
           this.sRegChal = null;
           this.bRespReceived = false;
           this.sRegResp = null;
           this.bDemoExpired = false;
           this.installDate = System.currentTimeMillis();
    }//kablogPrefsRecord()
    
    public String toString() {
        return "PrefsRecord " +
             "\nURL: " +sURL+
            "\nUsername: " +sUsername+
            "\nPassword: " +sPassword + 
            "\nCur Blog ID: " +sCurBlogID + "\n";      
    }//toString
    
    /**
     * Write this record's data to the stream provided. (similar to serializing)
     */
    public void writeToStream(DataOutputStream os) {

        try {
            os.writeUTF(this.sURL);
            os.writeBoolean(this.bHostInfoEdited);
            os.writeInt(this.nBlogServerType);
            os.writeUTF(this.sUsername);
            os.writeUTF(this.sPassword);
            os.writeBoolean(this.bUserInfoEdited);
            os.writeUTF(this.sCurBlogID);
            os.writeBoolean(this.bBlogSelected);
            
            os.writeBoolean(this.bDemoExpired);
            os.writeLong(this.installDate);

            os.writeBoolean(this.bChalGenerated);
            os.writeBoolean(this.bRespReceived);
            
            //only write out the reg info if it's valid
            if (this.bChalGenerated) {
                os.writeUTF(this.sRegChal);
                if (this.bRespReceived) {
                    os.writeUTF(this.sRegResp);
                }
            }
       }
        catch (Exception ex) {
            System.err.println("write prefs ex: " +ex);
        }
    }//writeToStream
    
    /**
     * Read this record's data from an input stream.
     */
    public void readFromStream(DataInputStream is) {
      try {
            this.sURL = is.readUTF();
            this.bHostInfoEdited = is.readBoolean();
            this.nBlogServerType = is.readInt();
 
            this.sUsername = is.readUTF();
            this.sPassword = is.readUTF();
            this.bUserInfoEdited = is.readBoolean();
            this.sCurBlogID = is.readUTF();
            this.bBlogSelected = is.readBoolean();
            
            this.bDemoExpired = is.readBoolean();
            this.installDate = is.readLong();
            
            this.bChalGenerated = is.readBoolean();
            this.bRespReceived = is.readBoolean();
            if (this.bChalGenerated) {
                this.sRegChal = is.readUTF();
                if (this.bRespReceived) {
                    this.sRegResp = is.readUTF();
                }
            }
       }
        catch (IOException ioEx) {
            System.err.println("read prefs ioEx: " + ioEx);
        }       
    }//readFromStream
    
} //class kablogPrefsRecord