/* Copyright (c) 2001-2005 Todd C. Stellanova, rawthought
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The  above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE. 
 * 
 * Created on November 7, 2002, 6:02 PM
 */

package org.kablog.midlet2;

import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;

import org.kablog.kdb.*;
import org.kablog.kgui.*;
import org.kablog.xmlrpc.KDateConverter;

import java.util.*;


/**
 * A generic class for allowing the user to select multiple items
 * stored in the Blog entries db.
 *
 * @author  Todd C. Stellanova
 * @version 1.0
 */
public class MultiEntryPicker 
			extends Form 
			implements CommandListener, BlogViewChild {
    
    protected KViewParent viewParent;
    protected BlogMgr blogParent;
    protected EntriesDB db;
    protected Command cmd_Back;
    protected boolean[] selectedFlags;
    protected Hashtable entriesHash;
    protected ChoiceGroup choices = null;
 
    public final static boolean debugOn = false;
    
    public MultiEntryPicker(String title, EntriesDB myDB, KViewParent  myParent) {
        //super(title,Choice.MULTIPLE);
        super(title);
        setViewParent(myParent);
        db = myDB;
        rebuildDisplayList();
        
       cmd_Back = new Command("Back",Command.BACK,1);
       this.addCommand(cmd_Back);
    }//MultiEntryPicker
    
    /**
     * Handle ui events
     */
    public void commandAction(Command c, Displayable d) {
        if (c == cmd_Back) {
            viewParent.childFinished(this);
        }
    }//commandAction
    
    /**
     * Called whenever we need to rebuild the visible list of items
     */
    protected void rebuildDisplayList() {
        
        //clear any items that may already be in the list
        if (choices != null) {
            for (int i = 0; i < this.size(); i++) {
                if (this.get(i).equals(choices)) {
                    this.delete(i); //remove the choice group
                    break; //don't need to look further
                }
            }         
        }
        
        choices =null;
        
        
        try {
            db.resetSearch();
            EntryRecord entry = null;
            entriesHash = new Hashtable(db.getEntryCount());

            choices = new ChoiceGroup(null,Choice.MULTIPLE);
            while ( (entry = db.getNextEntry()) != null) {
                //Date storedDate = null;
                 String titleStr = entry.sTitle;
                 
                if (((titleStr == null) || (titleStr.length() == 0)) 
                    && ((entry.sBody != null) && (entry.sBody.length() > 0))) {
                        titleStr = entry.sBody;
                }
                int len = titleStr.length();
                int endIdx  = ( (len > 16) ? 16 : titleStr.length() );
               // if (debugOn) System.out.println("titleStr: " + titleStr);
                titleStr = titleStr.substring(0,endIdx);
               //  if (debugOn) System.out.println("titleStr: " + titleStr);

			  /*             
			  try { storedDate = KDateConverter.stringToDate(entry.sCreationDate,KDateConverter.DATE_TIME);}
			   catch (Throwable t) {}
			   finally { if (storedDate == null) storedDate = new Date(); }
			   String entryStr = KDateConverter.dateToIso8601String(storedDate,KDateConverter.DATE);
			    
			    entryStr += "\n" + titleStr;
			    */
            
               
               //System.out.println("### entryStr: " + entryStr);

                entriesHash.put(titleStr,new Integer(entry.getRecID()));//TODO store the whole entry instead?
                choices.append(titleStr,null);
            }//foreach db entry
        }
        catch (KDbException dbEx) {
            if (debugOn) System.err.println("search db failed: " + dbEx);
        }    
        
        if (choices != null) {
            //System.out.println("### total choices: " + choices.size());
            this.append(choices); //make ChoiceGroup visible
        }
        
                //deselect all to begin with
        //make certain that all of our menu items are deselected to begin with
        /*
        if (this.size() > 0) {
           this.selectedFlags = new boolean[this.size()];
           int numSelected = getSelectedFlags(selectedFlags);
           if (numSelected > 0) {
               for (int i = 0; i < selectedFlags.length; i++) {
                   if (debugOn) System.out.println("Found bad set flag: " +i);
                   selectedFlags[i] = false;
               }
               setSelectedFlags(selectedFlags);
           }
           //we leave selectedFlags hanging around in case our subclasses need to use that storage
        }//num menu items > 0
         */
    }//rebuildDisplayList
    
    
     /**
     * Build and return a list of IDs for all the entries that are selected
     */
    protected Vector getSelectedEntries() {
        Vector selectedIDs = new Vector(choices.size());
        selectedFlags = new boolean[choices.size()];
        int numSelected = choices.getSelectedFlags(selectedFlags);
       // if (debugOn) System.out.println("numSelected: " +numSelected);
        
        if (numSelected > 0) { //need at least one..select the first one
            for (int i = 0; i < selectedFlags.length; i++) {
                if (selectedFlags[i]) {
                    String key = choices.getString(i);//get the string associated with this list item
                    Integer recID = (Integer)entriesHash.get(key); //XXX store whole entry instead?
                    selectedIDs.addElement(recID);
                }
            }
        }
        return selectedIDs;

    }//getSelectedEntries
    
    /**
     * @param The db id of the entry to remove from the display list.
     */
    protected void removeEntryFromDisplayList(Integer recID) {
        for (int i = 0; i < choices.size(); i++) {
            String itemStr = choices.getString(i);
            Integer curRecID = (Integer)entriesHash.get(itemStr);
            if (curRecID == recID) {
                choices.delete(i);
                break;
            }
        }
    }
    
    /** @param The callback client interested in receiving finished status.
     */
    public void setViewParent(KViewParent parent) {
        this.viewParent = parent;
    }
    
    /** @param parent to set
     */
    public void setBlogParent(BlogMgr parent) {
        this.blogParent = parent;
    }
  
    public void doCleanup()
    {
    
    }  
//removeEntryFromDisplayList
    
}//class MultiEntryPicker
