/* Copyright (c) 2001-2005 Todd C. Stellanova, rawthought
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The  above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE. 
 * 
 * Created on October 31, 2002, 12:03 PM
 */

package org.kablog.midlet2;


import javax.microedition.lcdui.*;

import org.kablog.kgui.*;

import java.util.*;
import java.io.*;

/**
 * A base class for all of our setup (preferences) forms
 *
 * @author  Todd C. Stellanova
 */
public abstract class SetupFormBase extends Form implements CommandListener, KViewChild {
       
    protected KViewParent parent;
    protected Command cmd_Cancel; // Close without saving
    protected Command cmd_Save; //Save these settings and close
    protected static final boolean debugOn = false;
    
    /**
     * The SetupFormBase calls this method to allow subclasses a chance to
     * scrape input data from the input form before saving.
     */
    public abstract void collectFormData(KablogPrefs prefs); 
    public abstract void addFormItems(KablogPrefs prefs);
    
    
    /** Creates a new instance of kablogPrefsForm */
    public SetupFormBase(String titleStr, KablogMIDlet myParent) {
        super(titleStr);
        parent = myParent;
        KablogPrefsDB prefsDB = KablogPrefsDB.getDB();
         
        cmd_Cancel = new Command("Cancel", Command.SCREEN, 2);
        cmd_Save = new Command("Save", Command.SCREEN, 2);
        addCommand(cmd_Save);
        addCommand(cmd_Cancel);

        addFormItems(prefsDB.getKablogPrefsRecord());//give subclasses a chance to insert form elements
        
        setCommandListener(this);
    }//kablogPrefsForm
    
    /**
     * Close without saving.
     */
    protected void doClosePrefs() {  //XXX is this a no-op?
    }//doClosePrefs
    
    public void doCleanup()
    {
    
    }
    
    /**
     * Save and close
     */
    protected void doSavePrefs() {
        KablogPrefsDB prefsDB = KablogPrefsDB.getDB();        
        KablogPrefs prefs = prefsDB.getKablogPrefsRecord();
                       
        collectFormData(prefs);
        //if (debugOn) System.out.println("prefs are set to: " + prefs);
        prefsDB.flushPrefs();
    }//doSavePrefs
    
      /**
     * Respond to commands, including exit
     * On the exit command, cleanup and notify that the MIDlet has been destroyed.
     */
    public void commandAction(Command c, Displayable s) {
 
        if (c == cmd_Save) {
            doSavePrefs();
        }
        else if (c == cmd_Cancel) {
            doClosePrefs();
        }
        parent.childFinished(this);
    }//CommandAction
    
    public void setViewParent(KViewParent parent) {
     this.parent = parent;   
    }
} //SetupFormBase
