/* Copyright (c) 2001-2005 Todd C. Stellanova, rawthought
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The  above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE. 
 * 
 *
 * Created on November 15, 2002, 2:13 PM
 */

package org.kablog.midlet2;

import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;

import org.kablog.kgui.*;



/**
 *
 * @author  Todd C. Stellanova
 */
public class StatusDialog 
	extends Form 
	implements CommandListener, Runnable 
	{
    
	StringItem msgField;
    Command cmd_Cancel;
    KNetworkClient netClient;
    boolean bCancelled;
    KablogMIDlet parent;
    Gauge progressGauge;
    
    /**
     *
     */
    public StatusDialog(String title, String msg,  KNetworkClient target, 
        KablogMIDlet myParent, boolean allowCancel) {
        super(title);
        parent = myParent;
        netClient = target;
        append(msg);
       
       progressGauge = new Gauge(null,false,100,0);
       append(progressGauge);
       
       if (allowCancel) {
           cmd_Cancel = new Command("Cancel",Command.CANCEL,1);
           addCommand(cmd_Cancel);
       }
      
       bCancelled = false;
       setCommandListener(this);
       
       //Display.getDisplay(parent).callSerially(this);
       statusThread = new Thread(this);
       statusThread.start();
       
       System.out.println("<<< StatusDialog() ");
       
    }//StatusDialog
    
    /**
     * Process cancel events
     */
    public void commandAction(Command c, Displayable d) {
        if (c == cmd_Cancel) {
            this.bCancelled = true;
            //pass along cancel to the netClient
            if (netClient.isCancellable())
                netClient.cancel();
            
            if (null != statusThread) {
            	statusThread = null; //causes cleanup?
            }
            	
        }
    }//commandAction
    
    
    /**
     * From Runnable
     */
    public void run() {
        
    	displayStatusLoop();
    	
    } //run
    
    
    protected void displayStatusSerially() {
        //repeatedly do Display.callSerially until the cows come home
        if (!bCancelled) {
            int percentComplete = netClient.getProgress();
            int lastErr = netClient.getFailureCode();
                
            //System.out.println("percentComplete: " + percentComplete + " lastErr: " + lastErr);
           if ((percentComplete == 100) || (lastErr != 0)) {
                //the operation is finished, for good or bad
                parent.asyncOpComplete(netClient.getAsyncResult(),lastErr);
                netClient.reset();
                
            }
            else {
                //update the display
                progressGauge.setValue(percentComplete);
                Display.getDisplay(parent).callSerially(this);
                //give other nonsense a chance to run?
                //try {Thread.sleep(500);} catch (Throwable t) {}
            }
        } 	
    
    }
    
    protected void displayStatusLoop() {
    	
    	System.out.println(">>> displayStatusLoop ");
    	
    	while (!bCancelled) {
            int percentComplete = netClient.getProgress();
            int lastErr = netClient.getFailureCode();
            
            //if (bDebug) System.out.println("percentComplete: " + percentComplete + " lastErr: " + lastErr);
            if ((percentComplete == 100) || (lastErr != 0)) {
                parent.asyncOpComplete(netClient.getAsyncResult(),lastErr);
                netClient.reset();
                statusThread = null; //causes cleanup?
                break;
            }
            else {
            	//update display
                progressGauge.setValue(percentComplete);
                      
                try {Thread.sleep(1000);}
                catch (Throwable t) {
                	System.err.println("loop threw: " + t);
                	break;
                }
            }   		
    	}
 
    
      	System.out.println("<<< displayStatusLoop ");
     }

    Thread statusThread;
} //StatusDialog

