/* Copyright (c) 2001-2005 Todd C. Stellanova, rawthought
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The  above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE. 
 * 
 * 
 */

package org.kablog.xmlrpc;

import java.io.*;
import java.util.*;
import javax.microedition.io.*;

//used to report progress
//used to process special kinds of data
import org.kablog.kgui.KProgressStatus;
import org.kobjects.base64.*;


//generic xml stuff


import org.kxml2.io.*;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;




/**
 * A wholesome class for sending XMLRPC requests to an XMLRPC server
 * and receiving and processing the result responses into useful
 * Java objects.
 * 
 * @author Todd C. Stellanova
 */
public class KXmlRpcClient2 implements KProgressStatus {
    
    public static final String RPCNAME_GET_USERS_BLOGS = "blogger.getUsersBlogs";
    public static final String RPCNAME_POST_FULL_ENTRY = "metaWeblog.newPost";
    public static final String RPCNAME_POST_SIMPLE_ENTRY = "blogger.newPost";
    public static final String RPCNAME_POST_MEDIA_OBJ = "metaWeblog.newMediaObject";
    public static final String RPCNAME_MT_GET_CATLIST = "mt.getCategoryList";
    public static final String RPCNAME_MW_GET_RECENT_POSTS = "metaWeblog.getRecentPosts";

    protected int progressState = 0;
    
    protected Object 	asyncResult;
    
    /**
     * The complete URL to be used to connect to the XMLRPC server.
     */
    protected String      accessURL;
    
    /**
     * The current connection to the server.
     */
    protected HttpConnection serverCon; //javax.microedition.io.HttpConnection
    
    /**
     * Used to write XMLRPC data to the server.
     */
    protected KXmlSerializer xmlWriter;
    
    /**
     * Used to read XMLRPC response data.
     */
    protected KXmlParser xmlParser;
    protected Vector parseParams;

     /** 
     * Turns debugging on/off
     */
    protected static final boolean bDebug = false;

    /**
     * Constructs KXmlRpcClient that will access the server given by the URL.
     *
     * @param URL Complete URL for accessing the XMLRPC server.
     */
    public KXmlRpcClient2(String URL) {
	accessURL = URL;
    }//KXmlRpcClient
    
    /**
     * Set the access URL 
     * @param URL with which to access the XMLRPC server.
     */
    public void setTargetURL(String URL) {
        accessURL = URL;
    }
    /**
     * Executes a remote procedure call on the XMLRPC server.
     * Prior to calling this method, you must set the server's URL properly.
     *
     * @param method The name of the XMLRPC method to be called on the server.
     * @param params A list of params required by the method call.
     * @return The result returned by the server-- typically a Hashtable or null
     */ 
    public Object remoteCall( String method, Vector params) throws KXmlRpcFault, IOException { 

       this.asyncResult = null;
       int resultCode = 0;
       
        
        try {
            progressState = 0;
            resultCode = writeRequest(method,params);
            
            if (resultCode == HttpConnection.HTTP_OK) {
                progressState = 50;
                asyncResult = readResponse();
                progressState = 90;
            }
            else {
               if (bDebug)   System.out.println("writeReq rc: " + resultCode);
               asyncResult = new KXmlRpcFault(resultCode,"HTTP Error");
            }
        } 
        catch (KXmlRpcFault parseEx) {
            if (bDebug)  System.out.println("parseEx: " + parseEx);
            asyncResult = parseEx;
        }
        catch (ConnectionNotFoundException connEx) {
            if (bDebug)  System.out.println("remoteCall connection ex: " + connEx);
            asyncResult = connEx;
        }
        catch ( IOException ioEx ) {
            if (bDebug)  System.out.println("remoteCall ioEx: ");
            ioEx.printStackTrace ();
            asyncResult = ioEx;
        } 
        finally {
            try {
                if ( serverCon != null ) {
                	serverCon.close();
                	serverCon = null;
                }
            } catch ( IOException ioe ) {
            	if (bDebug)  System.out.println("Closing serverCon ioe: ");
                ioe.printStackTrace ();
            }
            catch (Exception ex) {
             	if (bDebug)  System.out.println("Closing serverCon ex: " +ex);
             }
        }//end try/catch/finally

        if (bDebug) System.out.println("remoteCall result: " + asyncResult);
        
	     if (asyncResult instanceof KXmlRpcFault) 
	        throw((KXmlRpcFault)asyncResult);
	     else if (asyncResult instanceof Exception)
	        throw ((IOException)asyncResult);
    
        return asyncResult;
    }// remoteCall


    /**
     * Write the HTTP request to the remote xmlrpc server.
     */
    protected int writeRequest(String method, Vector params) throws IOException {
 
        int rc;
        boolean bKeepGoing;
        String newURL;
        //create an output stream to write out the data on
        ByteArrayOutputStream bos = new ByteArrayOutputStream(
        		RPCNAME_POST_MEDIA_OBJ.equals(method) ? 4096 : 1024);//should be adequate for most requests
        
        progressState += 5;
        //if (bDebug) System.out.println("calling writeXmlRpcCall...");
        writeXmlRpcCall( method, params, bos );      
        //if (bDebug) System.out.println("...writeXmlRpcCall done");
        progressState += 5;
 
        //if (bDebug)  System.out.println( bos.toString () );
        byte[] requestData = bos.toByteArray();
        bos  = null; //no longer used
        newURL = this.accessURL;
        
        bKeepGoing = true;

        do {
   

            
            serverCon  = ( HttpConnection ) Connector.open( newURL, Connector.READ_WRITE );
            serverCon.setRequestMethod( HttpConnection.POST );
            serverCon.setRequestProperty( "Content-Type", "text/xml" );
            serverCon.setRequestProperty( "User-Agent", "kablog-j2me/2.0.9" );          
            serverCon.setRequestProperty( "Content-Length", Integer.toString( requestData.length ) );
            
             
            //
            //============ WARNING ======================
            //Right now the WTK emulator removes the Content-Length header above 2kB
            //and switches to chunked transfer-encoding.  Unfortunately, Apache and some other
            //web servers don't like that.  This switch appears to violate the intent of using chunking,
            //since we know and set the Content-Length, but the WTK HTTP implementation chooses to
            //discard that information. 
            //
            //We set the Content-Length header and assume the HttpConnection does the right thing-- which
            //it does on most handsets, just not the reference J2ME-WTK implemenation.
            //
            
            //The following code checks some props related to this WTK issue
            //if "use_chunks' is turned on, it appears there's no way to turn it off
            //the "output_buffer_size" dictates how the size at which the httpconn switches over to chunking
            
            if (bDebug) {
                System.out.println("=== Sun chunkProp: " + System.getProperty("com.sun.midp.io.http.use_chunks"));        
                System.out.println("=== Sun sizeProp: " + System.getProperty("com.sun.midp.io.http.output_buffer_size"));
            }
          
     
            //Sometimes we need to make sure the Host property is set correctly: otherwise, on a server
            //that serves multiple virtual domains, it wouldn't know which Host to serve.
            String trueHost = serverCon.getHost();
           
             if (trueHost == null) {
                if (bDebug) 
                	System.out.println("=== invalid Host !");
                
                int dblSlashPos = newURL.indexOf("//");
                if (dblSlashPos > -1) {
                    String newHost;
                    int dirSlashPos = newURL.indexOf("/",dblSlashPos+2);
                    if (dirSlashPos > -1) {
                        newHost = newURL.substring(dblSlashPos+2,dirSlashPos);
                    }
                    else newHost = newURL.substring(dblSlashPos+2);
                    
                    serverCon.setRequestProperty("Host",newHost);
                   if (bDebug) System.out.println("-- newHost: " + newHost);
               }
            }
             
 
			// Write the request to the server.
             if (bDebug)
             {
             	System.out.println("Content-Length: " + serverCon.getRequestProperty("Content-Length"));
            	System.out.println("Transfer-Encoding: " + serverCon.getRequestProperty("Transfer-Encoding"));
             }

			OutputStream out = serverCon.openOutputStream();
			out.write(requestData,0,requestData.length);
			//out.flush(); // Optional, getResponseCode will flush as well, but chunking behavior may be different?
			            
             if (bDebug)
             {
              System.out.println("after Content-Length: " + serverCon.getRequestProperty("Content-Length"));
              System.out.println("after Transfer-Encoding: " + serverCon.getRequestProperty("Transfer-Encoding"));
             }               
                           
            try {
            	rc  = serverCon.getResponseCode();
                if (bDebug) System.out.println("response code: " + rc);
            }
            catch (IOException respCodeEx) {
                if (bDebug)  System.err.println("getResponseCode ex: " +respCodeEx);
                rc = 666;
            }
            
            switch( rc ) {
                case HttpConnection.HTTP_MOVED_PERM:
                case HttpConnection.HTTP_MOVED_TEMP:
                case HttpConnection.HTTP_SEE_OTHER:
                case HttpConnection.HTTP_TEMP_REDIRECT:
               	if (bDebug) System.out.println("## redir rc: " + rc);
                newURL = serverCon.getHeaderField( "Location" );
                  if( newURL != null && newURL.startsWith( "/*" ) ){
                    StringBuffer b = new StringBuffer();
                    b.append( "http://" );
                    b.append( serverCon.getHost() );
                    b.append( ':' );
                    b.append( serverCon.getPort() );
                    b.append( newURL );
                    newURL = b.toString();
                  }
                  
                  if (bDebug) System.out.println("## newURL: " + newURL);
                  out.close();
                  out = null;
                  serverCon.close();
                  serverCon = null;
                  break;
                  
                case HttpConnection.HTTP_OK:
                default:
                    newURL = null;
                    bKeepGoing = false;
                    break;
            } //switch rc
        
        } while (bKeepGoing);
    
		
        progressState += 5;

        return rc;
    }//writeRequest
    
    

    /**
     * Read response from server, parse it, and return results.
     *
     */
    protected Object readResponse() throws KXmlRpcFault, IOException {
		InputStream   in = null;
		Object result = null;
		long availLen = 0;

	  
	     in = serverCon.openInputStream ();
	     availLen = serverCon.getLength();
	     System.out.println("## content len: " +availLen);
         
          if (availLen < 0) {
             int headAvail = serverCon.getHeaderFieldInt("Content-Length",-666);
            //System.out.println("## headAvail: " +headAvail);
            if (headAvail > 0) availLen = headAvail;
            else if (headAvail < 0) availLen = 999999; //XXX ludicrously large data
             
         }
          
        if (!(availLen > 0))  
             return null;
        progressState += 5;

        // Parse response from server
        if (in != null) {
            progressState += 5;
            result = parseXmlRpcResponse(in);
            progressState += 5;
        }
        else {
            if (bDebug)  System.out.println("Failed to get response from server.");
            throw new IOException("openInputStream failed");
            }
        
     
        return result;
    }//readResponse
    
    

    //-- Methods for writing requests to the server
     /**
     * Builds an XMLRPC document (consisting of XML data).
     *
     * @param methodName The name of the remote procedure to call.
     * @param params Parameters to be passed into the remote procedure call.
     */
    protected void writeXmlRpcCall( String methodName, Vector params, OutputStream bos ) throws IOException {
            
    	this.xmlWriter = new KXmlSerializer();
    	xmlWriter.setOutput(bos,"UTF-8");
  
		//start <methodCall>
		xmlWriter.startTag( null,"methodCall" );
		//start <methodName>
		xmlWriter.startTag( null, "methodName" );
		xmlWriter.text(methodName);
		xmlWriter.endTag(null, "methodName");// end </methodName>

        //if params is non-null write params to the server
        if (params != null) {
            //start <params> if it's non-null (it might be empty?)
            xmlWriter.startTag(null,"params");
            for (int i = 0; i<params.size(); i++) {
                xmlWriter.startTag(null,"param"); //start <param>
                writeXmlRpcValue(params.elementAt(i));//write out the value of the given param
                xmlWriter.endTag(null,"param"); //end </param>
            }
            xmlWriter.endTag(null,"params"); //end </params> 
        }
	
		xmlWriter.endTag(null,"methodCall");//end </methodCall>
        
        xmlWriter.flush(); //make sure the entire atomic call is flushed out the pipe
        xmlWriter = null; //release writer when finished
    }//writeXmlRpcCall
    
    
/**
 * Marshalls Java values into XMLRPC-formatted xml data.
 */
private void writeXmlRpcValue( Object value ) throws IOException {
    String valStr;
    //start <value>

	xmlWriter.startTag( null,"value" );

    //place "lengthiest" types at the top of this list
     
    // Hashtable is shipped as an XMLRPC <struct> 
    // can have an arbitrary number of fields 
    
	if( value instanceof Hashtable ) {
	    Hashtable h = (Hashtable) value;
	    xmlWriter.startTag( null,"struct" );//start <struct>
	    for( Enumeration e = h.keys(); e.hasMoreElements(); ) {
			Object key = e.nextElement();
			xmlWriter.startTag( null,"member" );//start <member>
	                writeWholeXmlTag("name",key.toString()); //<name>foo</name>
	                //now recurse to write out the value associated with this key (name)
	                
	       // if (bDebug) System.out.println("writeVal: " + key.toString());
	        
			writeXmlRpcValue( h.get(key) );
			xmlWriter.endTag(null,"member");//end  </member>
	    }//foreach key
            
        xmlWriter.endTag(null,"struct"); //</struct>
	}
    else if( value instanceof Vector ) { //encode java Vectors as XMLRPC arrays
	    Vector v = (Vector) value;
	    xmlWriter.startTag( null,"array" ); //<array>
	    for( int i = 0; i < v.size(); i++ ) {
                Object curObj = v.elementAt(i);
                //recursively write all the items in the array
                writeXmlRpcValue(curObj );
            }//foreach item in array
            xmlWriter.endTag(null,"array");// </array>
	}
    else if( value instanceof String ) {
            writeWholeXmlTag("string",(String)value);//<string>foobar</string>
	}
    else if( value instanceof byte[] ) {
            //encode byte arrays as base64
            if (bDebug) System.out.println("Writing byte val: " + ((byte[]) value).length);;
            try { 
            	 byte[] dataBlock = (byte[]) value;
            	System.gc();
				StringBuffer stringBuf =  new StringBuffer((dataBlock.length * 3 ) /2 );
				Base64.encode( dataBlock, 0, dataBlock.length,stringBuf);

            	try { 
            		if (bDebug) System.out.println("stringBuf cap: " + stringBuf.capacity() + " len: " + stringBuf.length());
	            	//valStr = stringBuf.toString();
	            	valStr = null;
	            	
	            	writeWholeXmlTag_Safe("base64",stringBuf);//<base64>foobar</base64>
	            	stringBuf = null;
            	}
            	catch  (java.lang.OutOfMemoryError oomErr)
            	{
            		if (bDebug) System.err.println("### oom on base64 write:  " + Runtime.getRuntime().freeMemory());
            	}
            }
            catch (java.lang.OutOfMemoryError oom) {
            	if (bDebug) System.err.println("### oom on base64 encode: " + Runtime.getRuntime().freeMemory());
            }
             
            if (bDebug) System.out.println("...byteval done");
	}
    else if( value instanceof Date ) {
            //encode dates in ISO8601 format
            valStr = KDateConverter.dateToIso8601String ( (Date) value, KDateConverter.DATE_TIME );
            writeWholeXmlTag("dateTime.iso8601",valStr);//<dateTime.iso8601>foobar</dateTime.iso8601>
	}
	else if( value instanceof Integer ) {
            valStr = ((Integer)value).toString();
            writeWholeXmlTag("int",valStr); //we could also use <i4> but this seems cleaner ?
	}
	else if( value instanceof Boolean ) {
            valStr = ( ( (Boolean)value ).booleanValue() ? "1" : "0");
            writeWholeXmlTag("boolean",valStr);
	}
 	else {
        //We are sloppy and loose here:
        //We encode unknown objects as strings.
        //This gets interesting when we're talking about float values
        //such as doubles-- if the ROM supports doubles, then Double.toString
        //should return the right thing....right?
        //if (bDebug) System.out.println("writeXmlRpcValue: unknown type: " +value);
        writeWholeXmlTag("string",value.toString());
     }

	xmlWriter.endTag(null,"value");//end </value>
    
}//writeXmlRpcValue
    
    /**
     * Handy method for writing out a complete <starttag>value</endtag>
     * Assumes xmlWriter is already running
     */
    protected void writeWholeXmlTag(String tagType, String value) throws  IOException {
        xmlWriter.startTag(null,tagType); //start <tagname>
        xmlWriter.text(value);
        xmlWriter.endTag(null,tagType);//end </tagname>
    }//writeWholeXmlTag
    
    protected void writeWholeXmlTag_Safe(String tagType, StringBuffer valBuf) throws IOException {
    	char[] charBlock = new char[256];
    	int srcBegin, srcEnd, total = valBuf.length(), remain;
    	
        xmlWriter.startTag(null,tagType); //start <tagname>
 
 		for (srcBegin = 0, remain=total; srcBegin < total; )
 		{
 			int blockSize = (remain < 256) ? remain: 256;
	 		srcEnd = srcBegin + blockSize;
	 		//if (bDebug) System.out.println("getchars (" + srcBegin + ", " + srcEnd );
	 		valBuf.getChars(srcBegin, srcEnd,charBlock,0);

			srcBegin += blockSize;
			remain -= blockSize;
			try {xmlWriter.text(charBlock,0,blockSize);}
			catch (java.lang.OutOfMemoryError oom)
			{
			 	if (bDebug) System.err.println("### oom on block xml write:  " + Runtime.getRuntime().freeMemory());
			 	break;
			}
		}
		
         xmlWriter.endTag(null,tagType);//end </tagname>

    }
     
    //-- Methods for reading response from XMLRPC server
    
    
    
    /**
     * Parse a single XMLRPC <fault>
     */
    public KXmlRpcFault parseXmlRpcFault() throws XmlPullParserException, IOException {
    	
        int fCode = -1;
        String stringy = "Unknown XMLRPC Fault";
  	
    	
    	if (xmlParser.START_TAG == xmlParser.nextTag())
    	{
    		
    		if ("value".equals(xmlParser.getName())) {  		
		      // XmlRPC faults are returned as structs (which are mapped to Hashtables)
				Hashtable fault = (Hashtable) parseXmlRpcValue();
				
		         // Ultimately, a client-side exception object is generated
		        
		        try { 
		            stringy = (String)fault.get("faultString");
		            if (stringy.indexOf("login") >= 0 )
		                fCode = KXmlRpcFault.UNAUTHORIZED;
		            else if (stringy.indexOf("Invalid blog") >= 0)
		                fCode = KXmlRpcFault.BAD_TARGET;
		            else if (stringy.indexOf("UserHasNoBlogsException") > 0)
		                fCode = KXmlRpcFault.USER_HAS_NO_BLOGS;
		            else
		                fCode = ((Integer) fault.get( "faultCode" ) ).intValue(); 
		        }
		        catch (Throwable baddy) {
		           if (bDebug) System.err.println("Could not obtain faultCode.");
		        };
    		}
        
    	}
        
        return new KXmlRpcFault ( fCode, stringy);
    }
    
     /** 
     * Called by a client to parse an XMLPC response returned by the server.
     *
     * @return The response parameter sent by the server.
     */
    public Object parseXmlRpcResponse(InputStream in) throws KXmlRpcFault, IOException {
    	
        Object          result = null;
        
        
        try {
        	     
	        this.xmlParser = new KXmlParser();	        
	        this.xmlParser.setInput(in, "UTF-8");
	        this.parseParams = new Vector();
	        
	        xmlParser.nextTag();
	        xmlParser.require(XmlPullParser.START_TAG, null, "methodResponse");
	        
	        	        
	        if (XmlPullParser.START_TAG == xmlParser.nextTag()) {
	           String tag = xmlParser.getName();
	           
	           if("fault".equals(tag)) {
	        	  
	        	   throw(parseXmlRpcFault()); 	   
	       
	           }
	           else if( "params".equals( tag ) ) {
		       		parseXmlRpcParams();
		            //XXX should we ever allow params.size > 1 ?
					if( parseParams.size() > 1 ) 
					    throw new IOException( "Too many return parameters" );
					else if( parseParams.size() == 1 ) 
					    result = parseParams.elementAt(0);
			   }
			   else 
				   throw new KXmlRpcFault(KXmlRpcFault.PARSE_FAILURE, "<fault> or <params> expected, got: " + tag );	        	    
	           
	        } 

        }
        catch (XmlPullParserException parseEx)
        {      	
        	System.out.println("PullParseEx: " + parseEx);      	
        	throw (new KXmlRpcFault(parseEx));
        }
        finally {
        	
        	xmlParser = null; //release when finished
        }
       
    
        
        return result;
    }//parseXmlRpcResponse



    /**
     * All data in an XML-RPC call is passed as a parameter. This method parses 
     * the parameter values out of each parameter by calling the parseXmlRpcValue() 
     * method. 
     */
    void parseXmlRpcParams() throws IOException, XmlPullParserException {
    	
    	while ( XmlPullParser.START_TAG == xmlParser.nextTag()) {

              		
            if ("param".equals(xmlParser.getName()))
            {
            	if ( XmlPullParser.START_TAG  == xmlParser.nextTag())
             	{
            		if ("value".equals(xmlParser.getName()))
            		{
            			Object parmVal = parseXmlRpcValue();
            			parseParams.addElement(parmVal);	
            		}
            		else 
            			break;
            	}
            }
            else
            	break;   		
    	}
    	
           

 }//parseXmlRpcParams
	    
 /** 
  * Parse XMLRPC values into Java objects. 
  * 
 * @return  Object representation of the XMLRPC value
 */

Object parseXmlRpcValue() throws IOException, XmlPullParserException {
	
	Object    result = null;
      
   
	while (XmlPullParser.START_TAG == xmlParser.nextTag()) {
		
	   String tag = xmlParser.getName();
	   
	   if ("struct".equals(tag))
		   result = parseXmlRpcStruct();
	   else if ("array".equals(tag))
		   result = parseXmlRpcArray();
	   else if ("base64".equals(tag))
		   result = Base64.decode(xmlParser.nextText());
	   else if ("int".equals(tag) || "i4".equals(tag))
		   result = Integer.valueOf(xmlParser.nextText());
	   else if ("boolean".equals(tag))
		   result = new Boolean(xmlParser.nextText().trim().equals("1"));
	   else if ("dateTime.iso8601".equals(tag))
		   result = KDateConverter.stringToDate(xmlParser.nextText(),KDateConverter.DATE_TIME);
	   //else if ("double".equals(tag))
		//   result = xmlParser.nextText();
	   //else if ("string".equals(tag))
		//   result = xmlParser.nextText();
	   else {
		   // "double" "string" or unknown....
			   result = xmlParser.nextText();
		   }
   }
        	   

	return result;
}//parseXmlRpcValue


 /**
 * Maps XML-RPC structs to java.util.Hashtables
 */  
  void  parseXmlRpcStructMember(Hashtable h) throws IOException, XmlPullParserException {
	  
	  	
	  	
	  	while ( XmlPullParser.START_TAG ==  xmlParser.nextTag()) {
        	
        	if ("name".equals(xmlParser.getName())) {
        		
        		String memberName = xmlParser.nextText();
        		Object memberVal = null;
        		       		
        		if (XmlPullParser.START_TAG == xmlParser.nextTag())
        		{
        			if ("value".equals(xmlParser.getName()))
        			{
		        		memberVal = parseXmlRpcValue();		        		
		        	}
        		}
        		
        		if (bDebug) System.out.println("## member name: " + memberName + " value: " + memberVal);
        		h.put(memberName,memberVal);
        	}       	
        	else 
        		break; //unknown struct member?
        		
        }	  	
  
  }//parseXmlRpcStructMember
  
  
  /**
   * Parse an XMLRPC <struct> into a Java object representation.
   * 
   * @return Hashtable representing struct member name / member value pairs
   * @throws IOException
   * @throws XmlPullParserException
   */
  Hashtable parseXmlRpcStruct() throws IOException, XmlPullParserException {
    	
	Hashtable h = new Hashtable();
		
   	while (XmlPullParser.START_TAG ==  xmlParser.nextTag()) {

   		if ("member".equals(xmlParser.getName())) {
        	parseXmlRpcStructMember(h);
        }
	
   	}

   	
   		/*
        while(true) {
            int eventType = xmlParser.nextTag();
            if (eventType == XmlPullParser.START_TAG) {
                String tag = xmlParser.getName();
                
                if ("member".equals(tag)) {
                	parseXmlRpcStructMember(h);
                }              
            }
            else
            	break;
        }
        */

		return h;
  }//parseXmlRpcStruct


    /**
     * Maps XML-RPC arrays to a Java Vector
     * 
     * @return Vector  representing XMLRPC <array>
     */
    Vector parseXmlRpcArray() throws IOException, XmlPullParserException {
 		Vector v = new Vector();
		
		if (XmlPullParser.START_TAG == xmlParser.nextTag())
		{		
		    //20021027 TCS: Need to skip <data> tag!
			if ("data".equals(xmlParser.getName()))
			{
				while (
						(XmlPullParser.START_TAG == xmlParser.nextTag()) &&
						 "value".equals(xmlParser.getName())
					 )
				{				
					v.addElement(parseXmlRpcValue());
				}	
			}   			
		}
		
		
		return v;
    }//parseXmlRpcArray
    
      //--- Methods for supporting KProgressStatus
    /** @ return Zero if no failure, or the nonzero error code;
     */
    public int getFailureCode() {
        int fcode = 0;
        
        
        if (asyncResult != null) {
            if (asyncResult instanceof KXmlRpcFault) {
                fcode  = ((KXmlRpcFault)asyncResult).code;
                if (fcode == KXmlRpcFault.UNAUTHORIZED)
                    fcode = KProgressStatus.ERR_BAD_USER;
                else if (fcode == KXmlRpcFault.USER_HAS_NO_BLOGS)
                    fcode = KProgressStatus.ERR_USER_HAS_NO_BLOGS;
            }
            else if (asyncResult instanceof Exception) {
                fcode = KProgressStatus.ERR_IO;
            }
        }
        
  
         return fcode;
        
    }//getFailureCode
    
    /** @return The progress of this item, from 0 to 100
     */
    public int getProgress() {
        //System.out.println("xml progress state: " + progressState);
        return progressState;
    }
    
    /** @return True if operation can be cancelled, false if not
     */
    public boolean isCancellable() {
        return false; //none of the operations can be cancelled currently.
    }
    
    /** Attempt to cancel the operation in progress. This may not work
     * if isCancellable returns false.
     */
    public void cancel() {
    }
    
    /** 
     * reset progress to zero
     */
    public void reset() {
        progressState = 0;
        asyncResult = null;
    }
    
}//XmlRpcclient

/*
class DbgInputReader extends InputStreamReader
{
    public DbgInputReader (InputStream in) {
        super(in);
    }
    
   public int read(char[] cbuf, int off, int len) throws IOException {
       int numRead = super.read(cbuf,off,len);
       System.out.println(" read: " + new String(cbuf));
       if (numRead > 0) {
           String dataStr = new String(cbuf,off,numRead);
           System.out.println("## data: " + dataStr);
       }
       
       return numRead;
   }
   
   public int read(char[] cbuf)  throws IOException {
       return this.read(cbuf,0,cbuf.length);
   }
   
   public int read() throws IOException {
       int val = super.read();
       System.out.print((char)val);
       return val;
       
   }

}//DbgInputReader
 */
 
