
/* Copyright (c) 2001-2005 Todd C. Stellanova, rawthought
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The  above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE. 
 * 
 * 
 */

package org.kablog.xmlrpc;

import org.xmlpull.v1.XmlPullParserException;


/**
 * When XMLRPC goes wrong, it uses faults to indicate a problem.
 * 
 * @author Todd C. Stellanova
 */ 
public class KXmlRpcFault extends Exception {
    
    /**
     * These three fields represent three potential error levels
     */
    public static final int NONE = 0;
    public static final int RECOVERABLE = 1;
    public static final int FATAL = 2;
    public static final int BAD_TARGET = 555;
    public static final int UNAUTHORIZED = 666;
    public static final int USER_HAS_NO_BLOGS = 777;
    public static final int PARSE_FAILURE = 999;


    /**
     * The fault code of the exception.
     */
    public final int code;

    /**
     * This is the sole constructor for the KxmlRpcException class.
     * @param code an integer representing the error code
     * @param message a String containing the error message
     */    
    public KXmlRpcFault(int code, String msg) {
		super(msg );
		this.code = code;
    }//end KXmlRpcFault( int, String )
    
   public KXmlRpcFault(XmlPullParserException parseEx) {
	   super("Parse Exception: " + parseEx.getMessage() );
       this.code = PARSE_FAILURE;
    }

}//class KXmlRpcFault